package com.bull.cimero.pluginEditor.actions;

import org.eclipse.gef.ui.actions.ActionBarContributor;
import org.eclipse.gef.ui.actions.DeleteRetargetAction;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.gef.ui.actions.RedoRetargetAction;
import org.eclipse.gef.ui.actions.UndoRetargetAction;
import org.eclipse.gef.ui.actions.ZoomComboContributionItem;
import org.eclipse.gef.ui.actions.ZoomInRetargetAction;
import org.eclipse.gef.ui.actions.ZoomOutRetargetAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.actions.ActionFactory;

/**
 * this class is used to permit actions.
 * undo, redo, delete, zoom
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class DiagramActionBarContributor extends ActionBarContributor {

   /**
    * method to build actions.
    * @see org.eclipse.gef.ui.actions.ActionBarContributor#buildActions()
    */
	@Override
	protected final void buildActions() {
      addRetargetAction(new UndoRetargetAction());
      addRetargetAction(new RedoRetargetAction());
      addRetargetAction(new DeleteRetargetAction());
      //add the zoom action
      addRetargetAction(new ZoomInRetargetAction());
      addRetargetAction(new ZoomOutRetargetAction());

   }

	/**
	* method to declare globals actions key.
	* @see org.eclipse.gef.ui.actions.ActionBarContributor
	* #declareGlobalActionKeys()
	*/
	protected final void declareGlobalActionKeys() {
	}


	/**
	* method to add actions to the toolbar.
	* @param toolBarManager : the current toolbar
	* @see org.eclipse.ui.part.EditorActionBarContributor
	* #contributeToToolBar(org.eclipse.jface.action.IToolBarManager)
	*/
	@Override
	public final void contributeToToolBar(
			final IToolBarManager toolBarManager) {
      super.contributeToToolBar(toolBarManager);
	  toolBarManager.add(getAction(ActionFactory.UNDO.getId()));
      toolBarManager.add(getAction(ActionFactory.REDO.getId()));
      //add the zoom
      toolBarManager.add(new Separator());
      toolBarManager.add(new ZoomComboContributionItem(getPage()));
   }

	/**
	* method to add actions to the menu.
	* @param menuManager : the current menu
	*/
	@Override
	public final void contributeToMenu(final IMenuManager menuManager) {
		super.contributeToMenu(menuManager);
		//add a "Zoom" menu after "Edit"
        MenuManager zoomMenu = new MenuManager("Zoom");
        zoomMenu.add(getAction(GEFActionConstants.ZOOM_IN));
        zoomMenu.add(getAction(GEFActionConstants.ZOOM_OUT));
        //insertion
        menuManager.insertAfter(IWorkbenchActionConstants.M_EDIT, zoomMenu);
	}
}
