package com.bull.cimero.pluginEditor.editors.figure.BCFigure;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FigureUtilities;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LayoutManager;
import org.eclipse.draw2d.PolygonDecoration;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * This class is the generique BC figure.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public abstract class BCCimeroFigure extends INOUTFigure {

	/**the arrow size.*/
	private static final int SIZEARROW = 2;
	/** The dimension of this figure. */
	private Dimension size;
	/** The figure which represent the server. */
	private	ServeurLabel serveurLabel = null;
	/** The figure which represent the canal. */
	private ImageCanal imageCanal = null;
	/** arrow input. */
	private Figure arrowInput = null;
	/** arrow output. */
	private Figure arrowOutput = null;

	/**
	 * The copy constructor.
	 * @param nameServeur : the name of the serveur
	 * @param figure : path to the figure
	 * @param errorFigure : path to the error figure
	 * @param icon : path to the icon
	 */
	public BCCimeroFigure(final String nameServeur
			, final String figure, final String errorFigure
			, final String icon) {
		//add the serveur label
		add(getServeurLabel(nameServeur));
		//add the image canal
		add(getImageCanal(figure, errorFigure));
		setPathToFigure(figure);
		setPathToErrorFigure(errorFigure);
		setPathToIcon(icon);
	}

	/**
	 * return the instance of the Image Canal figure.
	 * @param imagePath : the image path
	 * @param imagePathError : the error image path
	 * @return ImageCanal
	 */
	private ImageCanal getImageCanal(
			final String imagePath
			, final String imagePathError) {
		if (imageCanal == null) {
			imageCanal = new ImageCanal(imagePath, imagePathError);
		}
		return imageCanal;
	}

	/**
	 * return the instance of the Image Canal figure.
	 * @param bErrors : boolean : show or not errors
	 * @return ImageCanal
	 */
	private ImageCanal getImageCanal(final boolean bErrors) {
		if (bErrors) {
			imageCanal.setErrorImage();
		} else {
			imageCanal.setNormalImage();
		}
		return imageCanal;
	}

	/**
	 * return the serverLabel Figure.
	 * @param nameServeur : the name of this label
	 * @return IFigure
	 */
	private IFigure getServeurLabel(final String nameServeur) {
		if (serveurLabel == null) {
			serveurLabel = new ServeurLabel(nameServeur);
		}
		return serveurLabel;
	}

	/**
	 * This function is used to create the arrow between the labelServeur and
	 * the canalImage.
	 * @param sens : IN or OUT
	 * @return Figure
	 */
	private Figure createArrow(final boolean sens) {
		/* Creating the connection */
		PolylineConnection c = new PolylineConnection();
		c.setSize(SIZEARROW, SIZEARROW);
		ChopboxAnchor sourceAnchor = null;
		ChopboxAnchor targetAnchor = null;
		//
		if (sens) {
			sourceAnchor = new ChopboxAnchor(serveurLabel);
			targetAnchor = new ChopboxAnchor(imageCanal);
		} else {
			sourceAnchor = new ChopboxAnchor(imageCanal);
			targetAnchor = new ChopboxAnchor(serveurLabel);
		}

		c.setSourceAnchor(sourceAnchor);
		c.setTargetAnchor(targetAnchor);

		/* Creating the decoration */
		PolygonDecoration decoration = new PolygonDecoration();
		PointList decorationPointList = new PointList();
		decorationPointList.addPoint(0, 0);
		decorationPointList.addPoint(-SIZEARROW , SIZEARROW);
		decorationPointList.addPoint(-SIZEARROW , 0);
		decorationPointList.addPoint(-SIZEARROW , -SIZEARROW);
		decoration.setTemplate(decorationPointList);
		c.setSourceDecoration(decoration);
		c.setTargetDecoration(decoration);

		/* Adding labels to the connection */
//		ConnectionEndpointLocator targetEndpointLocator =
//		        new ConnectionEndpointLocator(c, true);
//		targetEndpointLocator.setVDistance(15);
//		Label targetMultiplicityLabel = new Label("1..*");
//		c.add(targetMultiplicityLabel, targetEndpointLocator);
//
//		ConnectionEndpointLocator sourceEndpointLocator =
//			new ConnectionEndpointLocator(c, false);
//		sourceEndpointLocator.setVDistance(15);
//		Label sourceMultiplicityLabel = new Label("1");
//		c.add(sourceMultiplicityLabel, sourceEndpointLocator);
//
//		ConnectionEndpointLocator relationshipLocator =
//			new ConnectionEndpointLocator(c,true);
//		relationshipLocator.setUDistance(10);
//		relationshipLocator.setVDistance(-20);
//		Label relationshipLabel = new Label("contains");
//		c.add(relationshipLabel,relationshipLocator);

		return c;
	}

	/**
	 * @param name : name to set
	 * @see com.bull.cimero.pluginEditor.editors.
	 * figure.GeneriqueCimeroFigure#setName(java.lang.String)
	 */
	public final void setName(final String name) {
		serveurLabel.setText(name);
	}

	/**
	 * @return Dimension : fixed size
	 */
	private Dimension getFixedSize() {
		if (size == null) {
			size = FigureUtilities.getStringExtents("W",
													 getFont());
			size.height = 2 * 15 + 3 * 17 / 2 + 1 + 2 + 1;
			size.width *= 10;
			size.width += 2 * 15;
			size.width += 1;
		}
		return size;
	}

	/**
	 * @return Dimension : dimension to get
	 * @param wHint : width
	 * @param hHint : height
	 */
	public final Dimension getPreferredSize(final int wHint, final int hHint) {
		return getFixedSize();
	}

	/** method called when refresh.
	 * @see org.eclipse.draw2d.Figure#layout()
	 */
	public final void validate() {
		LayoutManager layout = getLayoutManager();
		if (isInput()) {
			layout.setConstraint(serveurLabel, new Rectangle(
					new Point(getFigureDimension().width
							- serveurLabel.getPreferredSize().width - 15, 0)
							, serveurLabel.getPreferredSize()));
			layout.setConstraint(imageCanal, new Rectangle(
					new Point(15, 13), imageCanal.getPreferredSize()));
		} else {
			layout.setConstraint(serveurLabel, new Rectangle(
					new Point(15, 0), serveurLabel.getPreferredSize()));
			layout.setConstraint(imageCanal, new Rectangle(
					new Point(getFigureDimension().width
							- imageCanal.getPreferredSize().width - 15, 13)
							, imageCanal.getPreferredSize()));
		}
		//we add the connection between the to component
		if (isInput()) {
			if (arrowOutput != null) {
				remove(arrowOutput);
			}
			arrowOutput = null;
			if (arrowInput == null) {
				add(getArrowInput());
			}
		} else {
			if (arrowInput != null) {
				remove(arrowInput);
			}
			arrowInput = null;
			if (arrowOutput == null) {
				add(getArrowOutput());
			}
		}
		super.validate();
	}

	/**
	 * method used to get Figure dimension.
	 * @return Dimension : the wanted dimension
	 */
	@Override
	public final Dimension getFigureDimension() {
		//add the image preferredSize
		int x =  imageCanal.getPreferredSize().width;
		int y = imageCanal.getPreferredSize().height;
		//add the Label preferred size
		x += serveurLabel.getPreferredSize().width;
		if (serveurLabel.getPreferredSize().height > y) {
			y = serveurLabel.getPreferredSize().height;
		}
		//add the two port
		x += 15 * 2;
		//add the size of the arrow between the image and label
		x += 50;
		return new Dimension(x, y);
	}

	/**
	 * get arrow input.
	 * @return Figure : wanted figure
	 */
	public final Figure getArrowInput() {
		if (arrowInput == null) {
			arrowInput = createArrow(false);
		}
		return arrowInput;
	}

	/**
	 * get arrow output.
	 * @return Figure : wanted figure
	 */
	public final Figure getArrowOutput() {
		if (arrowOutput == null) {
			arrowOutput = createArrow(true);
		}
		return arrowOutput;
	}

	/**
	 * method used to separate valide component to others.
	 * @param valide : validation result
	 */
	@Override
	public final void setValideDraw(final boolean valide) {
		super.setValideDraw(valide);
		imageCanal = getImageCanal(isValideDraw());
	}
}
