package com.bull.cimero.pluginEditor.editors.figure.BCFigure;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FlowLayout;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.geometry.Dimension;

/**
 * This class permit to draw a figure which represent the serveur.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class ServeurLabel extends Figure {

	/** the label to print. */
	private Label label;
	/** default width. */
	private static final int DEFAULTWIDTH = 100;
	/** default height. */
	private static final int DEFAULTHEIGHT = 80;
	/** max width of caracteres in a figure. */
	private static final int MAXWIDTH = 7;
	/** max height of caracteres in a figure. */
	private static final int MAXHEIGHT = 2;

	/**
	 * default constructor.
	 * @param s : String to print
	 */
	public ServeurLabel(final String s) {
		String txt = stringTransform(s);
		label = new Label(txt);
		add(label);

		setOpaque(true);
		setBackgroundColor(ColorConstants.white);
		setBorder(new LineBorder(2));

		FlowLayout layout = new FlowLayout();
		layout.setMajorAlignment(FlowLayout.ALIGN_CENTER);
		setLayoutManager(layout);
	}

	/**
	 * get the prefered size.
	 * @param wHint : width
	 * @param hHint : height
	 * @see org.eclipse.draw2d.IFigure
	 * #getPreferredSize(int, int)
	 * @return Dimension : wnated dimension
	 */
	public final Dimension getPreferredSize(
				final int wHint, final int hHint) {
		Dimension	preferredSize = new Dimension(DEFAULTWIDTH, DEFAULTHEIGHT);
		return preferredSize;
	}

	/**
	 * set the text to print.
	 * @param text : string to set
	 */
	public final void setText(final String text) {
		String txt = stringTransform(text);
		label.setText(txt);
	}

	/**
	 * this function transform a text.
	 * @param s : the string to transform
	 * @return the new string.
	 */
	private String stringTransform(final String s) {
		String txt = "";
		try {
			String tmp = "";
			int cpt = 0;
			int posOfLastSpace = 0;
			int nbEnter = 0;
			boolean bStop = false;
			while (!bStop) {
				if (cpt >= s.length()) {
					//search end
					bStop = true;
					if (posOfLastSpace == 0) {
						//no space so cut this part
						txt += tmp;
					} else {
						//cut at the last space
						txt += tmp.substring(0, posOfLastSpace);
						txt += " \n";
						txt += tmp.substring(posOfLastSpace + 1, tmp.length());
					}
				} else {
					if (nbEnter > MAXHEIGHT) {
						bStop = true;
						txt += "...";
					} else {
						if (s.charAt(cpt) == ' ') {
							posOfLastSpace = tmp.length();
						}
						tmp += s.charAt(cpt++);
						if (tmp.length() == MAXWIDTH) {
							if (posOfLastSpace == 0) {
								//no space so cut this part
								txt += tmp;
								txt += " \n";
								nbEnter++;
								tmp = "";
							} else {
								//cut at the last space
								txt += tmp.substring(0, posOfLastSpace);
								txt += " \n";
								nbEnter++;
								tmp = tmp.substring(posOfLastSpace + 1
														, tmp.length());
							}
							posOfLastSpace = 0;
						}
					}
				}
			}
		} catch (Exception e) {
			//if error occured, then put the name directly
			txt = s;
		}
		return txt;
	}
}
