package com.bull.cimero.pluginEditor.editors.palette;

import it.eng.components.cimero.CustomSimpleFactory;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.palette.CombinedTemplateCreationEntry;
import org.eclipse.gef.palette.ConnectionCreationToolEntry;
import org.eclipse.gef.palette.PaletteContainer;
import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteEntry;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.palette.PaletteSeparator;
import org.eclipse.gef.palette.SelectionToolEntry;
import org.eclipse.gef.palette.ToolEntry;
import org.eclipse.gef.requests.SimpleFactory;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import cimeroEditor.CimeroEditorPlugin;

import com.bull.cimero.pluginEditor.editors.figure.GeneriqueCimeroFigure;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.BindingComponentModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.FTP;
import com.bull.cimero.pluginEditor.editors.model.BCModel.JMS;
import com.bull.cimero.pluginEditor.editors.model.BCModel.Mail;
import com.bull.cimero.pluginEditor.editors.model.BCModel.RSS;
import com.bull.cimero.pluginEditor.editors.model.BCModel.ScreenOutput;
import com.bull.cimero.pluginEditor.editors.model.BCModel.ServiceWeb;
import com.bull.cimero.pluginEditor.editors.model.BCModel.Timer;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Aggregator;
import com.bull.cimero.pluginEditor.editors.model.SEModel.MessageFilter;
import com.bull.cimero.pluginEditor.editors.model.SEModel.People;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Pipeline;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Routeur;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Splitter;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Transformeur;
import com.bull.cimero.pluginEditor.editors.model.SEModel.WireTap;
import com.bull.cimero.pluginEditor.utils.classLoader.CimeroClassLoader;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;
import com.bull.cimero.pluginEditor.utils.ressources.Ressources;

/**
 * This is the palette class. This class describe the component of the palette.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public final class PaletteRootCimero {

	/**Current factory package.*/
	private static final String CURRENTPACKAGE =
		"com.bull.cimero.pluginEditor.editors.figure";
	/**BC figure package.*/
	private static final String BCPACKAGE = "BCFigure";
	/**SE figure package.*/
	private static final String SEPACKAGE = "SEFigure";
	/**The suffixe figure.*/
	private static final String SUFFIXEFIGURE = "Figure";

	/**
	 * default constructor.
	 */
	private PaletteRootCimero() {
		//nothing to do
	}

	/**
	 * create the control group.
	 * @param root : root to use
	 * @return PaletteContainer : created palette
	 */
	private static PaletteContainer createControlGroup(final PaletteRoot root) {
		PaletteGroup controlGroup = new PaletteGroup("Control Group");
		List < PaletteEntry > entries = new ArrayList < PaletteEntry >();
		ToolEntry tool = new SelectionToolEntry();
		entries.add(tool);
		root.setDefaultEntry(tool);
		controlGroup.addAll(entries);
		return controlGroup;
	}
	private static PaletteContainer createCustomPalette(Ressources resources, String catId, String catDescription) {
		PaletteDrawer drawer = new PaletteDrawer(catDescription, null);
		//init
		Image img = null;
		List < PaletteEntry > entries = new ArrayList < PaletteEntry >();
		//
		//add transformer element
		//
	
		
			
			NodeList nodeList = resources.getXPathNodeSet("/COMPONENTS/COMPONENT[@category='"+catId+"']");
			Node node = null;
			NamedNodeMap attributes = null;
			String className  = null;
			Class clazz = null;
			ToolEntry  tool = null;
			String category = "";
			for (int i = 0; i < nodeList.getLength(); i++) {
				node = nodeList.item(i);				
				attributes = node.getAttributes();
				
				
				className = attributes.getNamedItem("class").getNodeValue();
				
				try{
					clazz = Class.forName(className);
				}catch (Exception e) {
					e.printStackTrace();
				}
				
				String pathToIcon = attributes.getNamedItem("paletteImage").getNodeValue();
				img = CimeroEditorPlugin.getImageDescriptor(pathToIcon).createImage();
				tool = new CombinedTemplateCreationEntry(
						attributes.getNamedItem("paletteName").getNodeValue(),
						attributes.getNamedItem("description").getNodeValue(),
						clazz,
						new CustomSimpleFactory(clazz,attributes.getNamedItem("id").getNodeValue()),
						ImageDescriptor.createFromImage(img), //small
						null//large
				);
				entries.add(tool);
			}
			//
			//add People element
			
			
	        //
			drawer.addAll(entries);
		
		return drawer;
		//
		//add People element
		/*
		img = getImage(People.class.getName().toString());
		ToolEntry  tool = new CombinedTemplateCreationEntry(
				"People",
				"Create a new 'People adapter'",
				People.class,
				new SimpleFactory(People.class),
				ImageDescriptor.createFromImage(img), //small
				null//large
		);
		entries.add(tool);
        //
		drawer.addAll(entries);
		return drawer;
		*/
		
	}
	/**
	 * This function is used to add to the palette the ServiceEngine components.
	 * @return PaletteContainer : wanted container
	 */
	private static PaletteContainer createServiceEngineDrawer() {
		PaletteDrawer drawer = new PaletteDrawer("Service Engine", null);
		//init
		Image img = null;
		List < PaletteEntry > entries = new ArrayList < PaletteEntry >();
		//
		//add transformer element
		//
		img = getImage(Transformeur.class.getName().toString());
		ToolEntry tool = new CombinedTemplateCreationEntry(
				"Transformer",
				"Create a new 'transformer'",
				Transformeur.class,
				new SimpleFactory(Transformeur.class),
				ImageDescriptor.createFromImage(img), //small
				null//large
		);
		entries.add(tool);
		//
		// add router element
		//
		img = getImage(Routeur.class.getName().toString());
		tool = new CombinedTemplateCreationEntry(
				"Router",
				"Create a new 'message router'",
				Routeur.class,
				new SimpleFactory(Routeur.class),
				ImageDescriptor.createFromImage(img), //small
				null//large
		);
		entries.add(tool);
		//
		//add tracer element
		//
		img = getImage(WireTap.class.getName().toString());
		// add tracer element
		tool = new CombinedTemplateCreationEntry(
				"Tracer",
				"Create a new 'wire-tap'",
				WireTap.class,
				new SimpleFactory(WireTap.class),
				ImageDescriptor.createFromImage(img), //small
				null//large
		);
		entries.add(tool);
		//
		//add enricher element
		//
		img = getImage(Pipeline.class.getName().toString());
		// add enrichisseur element
		tool = new CombinedTemplateCreationEntry(
				"WS Pipeline",
				"Create a new web service pipeline",
				Pipeline.class,
				new SimpleFactory(Pipeline.class),
				ImageDescriptor.createFromImage(img), //small
				null//large
		);
		entries.add(tool);
		//
		//add aggregator element
		//
		img = getImage(Aggregator.class.getName().toString());
		// add aggregator element
		tool = new CombinedTemplateCreationEntry(
				"Aggregator",
				"Create a new 'aggregator'",
				Aggregator.class,
				new SimpleFactory(Aggregator.class),
				ImageDescriptor.createFromImage(img), //small
				null//large
		);
		entries.add(tool);
		//
		//add splitter element
		//
		img = getImage(Splitter.class.getName().toString());
		// add splitter element
		tool = new CombinedTemplateCreationEntry(
				"Splitter",
				"Create a new 'splitter'",
				Splitter.class,
				new SimpleFactory(Splitter.class),
				ImageDescriptor.createFromImage(img), //small
				null//large
		);
		entries.add(tool);
        //
        //add MessageFilter element
        //
        img = getImage(MessageFilter.class.getName().toString());
        // add message Filter element
        tool = new CombinedTemplateCreationEntry(
                "MessageFilter",
                "Create a new 'message filter '",
                MessageFilter.class,
                new SimpleFactory(MessageFilter.class),
                ImageDescriptor.createFromImage(img), //small
                null//large
        );
        entries.add(tool);

		
        //
		drawer.addAll(entries);
		return drawer;
	}

	/**
	 * This function is used to add to the palette the ServiceEngine components.
	 * @return PaletteContainer : wanted container
	 */
	private static PaletteContainer createBindingComponentsDrawer() {
		PaletteDrawer drawer = new PaletteDrawer("Binding Components", null);
		List < PaletteEntry > entries = new ArrayList < PaletteEntry >();
		//init
		Image img = null;
		//
		//MAIL SERVER
		//
		img = getImage(Mail.class.getName().toString());
		ToolEntry tool = new CombinedTemplateCreationEntry(
				"Mail",
				"Create a binding component to a Mail server",
				Mail.class,
				new SimpleFactory(Mail.class),
				ImageDescriptor.createFromImage(img), //small
				null//large
		);
		entries.add(tool);
		//
		//WEB SERVICE
		//
		img = getImage(ServiceWeb.class.getName().toString());
		tool = new CombinedTemplateCreationEntry(
				"Web Service",
				"Create a binding component to a WebService",
				ServiceWeb.class,
				new SimpleFactory(ServiceWeb.class),
//				ServiceWeb.class,
//				new SimpleFactory(ServiceWeb.class),
				ImageDescriptor.createFromImage(img), //small
				null//large
		);
		entries.add(tool);
		//
		//FTP SERVER
		//
		img = getImage(FTP.class.getName().toString());
		tool = new CombinedTemplateCreationEntry(
				"FTP Server",
				"Create a binding component to a FTP server",
				FTP.class,
				new SimpleFactory(FTP.class),
				ImageDescriptor.createFromImage(img), //small
				null//large
		);
		entries.add(tool);
		//
		//RSS Flow
		//
		img = getImage(RSS.class.getName().toString());
		tool = new CombinedTemplateCreationEntry(
				"RSS flow",
				"Create a binding component to a RSS flow",
				RSS.class,
				new SimpleFactory(RSS.class),
				ImageDescriptor.createFromImage(img), //small
				null//large
		);
		entries.add(tool);
		//
		//SCREEN output
		//
		img = getImage(ScreenOutput.class.getName().toString());
		tool = new CombinedTemplateCreationEntry(
				"Screen output",
				"Create a binding component to the screen",
				ScreenOutput.class,
				new SimpleFactory(ScreenOutput.class),
				ImageDescriptor.createFromImage(img), //small
				null//large
		);
		entries.add(tool);
		//
		//JMS SERVER
		//
		img = getImage(JMS.class.getName().toString());
		tool = new CombinedTemplateCreationEntry(
				"JMS server",
				"Create a binding component to a JMS server",
				JMS.class,
				new SimpleFactory(JMS.class),
				ImageDescriptor.createFromImage(img), //small
				null//large
		);
		entries.add(tool);
		//
		//TIMER
		//
		img = getImage(Timer.class.getName().toString());
		tool = new CombinedTemplateCreationEntry(
				"Timer",
				"Create a binding component to a timer",
				Timer.class,
				new SimpleFactory(Timer.class),
				ImageDescriptor.createFromImage(img), //small
				null//large
		);
		entries.add(tool);
		//
		drawer.addAll(entries);
		return drawer;
	}

	/**
	 * This function is used to create the connection element in the palette.
	 * @return PaletteContainer : wanted container
	 */
	private static PaletteContainer createConnectionDrawer() {
		PaletteDrawer drawer = new PaletteDrawer("Connections", null);
		List < PaletteEntry > entries = new ArrayList < PaletteEntry >();
		//
		//add of the arrow
		//
		Image img = getImageFromPath(
				"icons" + File.separator + "ico_arrow.jpg");
		ToolEntry tool = new ConnectionCreationToolEntry(
				"Simple",
				"Create a simple connection",
				null,
				ImageDescriptor.createFromImage(img), //small
				null//large
		);
		entries.add(tool);
		//add a separator
		PaletteSeparator separator = new PaletteSeparator(
					"id.palette.separator");
		separator.setUserModificationPermission(
					PaletteEntry.PERMISSION_NO_MODIFICATION);
		separator.setVisible(true);
		entries.add(separator);

		drawer.addAll(entries);
		return drawer;
	}

	/**
	 * method used to create category.
	 * @param root : paletteroot
	 * @return Lst : list of category
	 */
	private static List < PaletteEntry > createCategories(
												final PaletteRoot root) {
		List < PaletteEntry > categories = new ArrayList < PaletteEntry >();
		//add the default category
		categories.add(createControlGroup(root));
		//add connections
		categories.add(createConnectionDrawer());
		//add Service engien
		categories.add(createServiceEngineDrawer());
		//add binding components
		categories.add(createBindingComponentsDrawer());
		Ressources ressource = new Ressources();
		String fullPath = CimeroEditorPlugin.getRessourcePath("/ressources/CustomComponents.xml");
		if (fullPath != null) {
			
			if (fullPath.substring(0, "file:".length()).compareTo("file:") == 0) {
				fullPath = fullPath.substring("file:".length());
			}
		
			ressource.setDocument(fullPath);
			NodeList nodeList = ressource.getXPathNodeSet("/COMPONENTS/CATEGORY");
			Node node = null;
			NamedNodeMap attributes = null;
			String catId = "";
			String catDescription = "";
			for (int i = 0; i < nodeList.getLength(); i++) {
				node = nodeList.item(i);				
				attributes = node.getAttributes();
				
				catId = attributes.getNamedItem("id").getNodeValue();
				catDescription = attributes.getNamedItem("description").getNodeValue();
				categories.add(createCustomPalette(ressource, catId, catDescription));
			}
			
		}
		
		return categories;
	}

	/**
	 * The function to create a palette.
	 * @return PaletteRoot : wanted palette
	 */
	public static PaletteRoot createPalette() {
		PaletteRoot paletteRoot = new PaletteRoot();
		paletteRoot.addAll(createCategories(paletteRoot));
		return paletteRoot;
	}

	/**
	 * method to get an image.
	 * @param pathToIcon : path to the icon
	 * @return the Image
	 */
	private static Image getImageFromPath(final String pathToIcon) {
		return CimeroEditorPlugin.getImageDescriptor(pathToIcon).createImage();
	}

	/**
	 * method to get an image.
	 * @param modelPath : path of the model
	 * @return the Image
	 */
	private static Image getImage(final String modelPath) {
		String pathToIcon = getPathToIcon(modelPath);
		return CimeroEditorPlugin.getImageDescriptor(pathToIcon).createImage();
	}

	   /**
	    * method used to get the path to the icon associated
	    * with a given model.
	    * @param modelName : model name
	    * @return String : path to the icon
	    */
	   public static String getPathToIcon(final String modelName) {
		   String fullPackageClass = CURRENTPACKAGE;
		   GeneriqueCimeroModel model = null;
		   try {
			   model = (GeneriqueCimeroModel) CimeroClassLoader
									.newInstance(modelName, new Object[] {});
		   } catch (Exception e) {
			   return "";
		   }
		   String simpleName = model.getClass().getSimpleName();
		   //test BC or SE
		   if (model instanceof BindingComponentModel) {
			   //BC component.
			   fullPackageClass += "." + BCPACKAGE;
		   } else {
			   //SE Component.
			   fullPackageClass += "." + SEPACKAGE;
		   }
		   fullPackageClass += "." + simpleName + SUFFIXEFIGURE;
		   //create the figure
		   GeneriqueCimeroFigure genFigure = null;
			if (model instanceof BindingComponentModel) {
				try {
					genFigure = (GeneriqueCimeroFigure) CimeroClassLoader
								.newInstance(fullPackageClass, new Object[] {});
				} catch (Exception e) {
					CimeroLog.errorLog("the BC figure"
							+ fullPackageClass + "cannot be create", e);
					return "";
				}
			} else {
				try {
					genFigure = (GeneriqueCimeroFigure) CimeroClassLoader
								.newInstance(fullPackageClass, new Object[] {});
				} catch (Exception e) {
					CimeroLog.errorLog("the SE figure"
							+ fullPackageClass + "cannot be create", e);
					return "";
				}
			}
			return genFigure.getPathToIcon();
	   }

}
