package com.bull.cimero.pluginEditor.editors.parts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.List;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.editparts.AbstractEditPart;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.requests.DropRequest;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import cimeroEditor.CimeroEditorPlugin;

import com.bull.cimero.pluginEditor.editors.figure.GeneriqueCimeroFigure;
import com.bull.cimero.pluginEditor.editors.figure.BCFigure.BCCimeroFigure;
import com.bull.cimero.pluginEditor.editors.figure.BCFigure.BCConcreteFigure;
import com.bull.cimero.pluginEditor.editors.figure.BCFigure.INOUTFigure;
import com.bull.cimero.pluginEditor.editors.figure.SEFigure.SEConcreteFigure;
import com.bull.cimero.pluginEditor.editors.model.Diagram;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.BindingComponentModel;
import com.bull.cimero.pluginEditor.editors.model.SEModel.ServiceEngineModel;
import com.bull.cimero.pluginEditor.editors.policies
	.NodeGraphicalNodeEditPolicy;
import com.bull.cimero.pluginEditor.editors.policies.NodeRemoveEditPolicy;
import com.bull.cimero.pluginEditor.editors.policies.NodeSelectionEditPolicy;
import com.bull.cimero.pluginEditor.utils.classLoader.CimeroClassLoader;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;
import com.bull.cimero.pluginEditor.utils.ressources.Ressources;

/**
 * The generiqueCimeroPart class.
 * All the Cimero part must to extends this class.
 * This class contain all the functionnalities to draw a figure,
 * make connection beetwen two figure.
 * This class is the only class that can make the bind beetween
 * the model and the figure
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class GeneriqueCimeroPart
					extends AbstractGraphicalEditPart implements
					PropertyChangeListener, NodeEditPart {

	//----------------------------------------------
	// CONSTANTES
	/**Figure package.*/
	private static final String FIGUREPACKAGE =
		"com.bull.cimero.pluginEditor.editors.figure";
	/**binding component figure package.*/
	private static final String BCPACKAGE = FIGUREPACKAGE + "." + "BCFigure";
	/**Service enfine figure package.*/
	private static final String SEPACKAGE = FIGUREPACKAGE + "." + "SEFigure";
	/**The se package prefixe.*/
	private static final String SEPREFIXEMODEL = "SEModel";

	// ---------------------------------------------
	// VARIABLE

	/** current diagram. */
	private Diagram diagram;
	/** variable to know if the part has to differentiate errors. */
	private boolean bShowErrors;
	/**The class name part of this instance.
	 * This variable is used to create the figure.*/
	private String classNameModel = "";

	/**
	 * copy constructor.
	 * @param diagramParam : the diagram
	 * @param bShow : boolean to show or not errors
	 * @param classNameModelParam : the model component name that create
	 * 			this part.
	 */
	public GeneriqueCimeroPart(
			final Diagram diagramParam, final boolean bShow,
			final String classNameModelParam) {
		super();
		this.classNameModel = classNameModelParam;
		this.diagram = diagramParam;
		setBShowErrors(bShow);
	}

	/**
	 * abstract function to get the figure.
	 * @return IFigure : the figure
	 */
	protected IFigure getCimeroFigure() {
		GeneriqueCimeroModel model = (GeneriqueCimeroModel)getModel();
		
		if (model.isCustomComponent()){
			String customerComponentId = model.getCustomComponentId();
			Ressources ressource = new Ressources();
			String fullPath = CimeroEditorPlugin.getRessourcePath("/ressources/CustomComponents.xml");
			if (fullPath != null) {
				
				if (fullPath.substring(0, "file:".length()).compareTo("file:") == 0) {
					fullPath = fullPath.substring("file:".length());
				}
			
				ressource.setDocument(fullPath);
				
				
				NodeList nodeList = ressource.getXPathNodeSet("/COMPONENTS/COMPONENT"); 
				Node node = null;
				NamedNodeMap attributes = null;
			
				String idTemp = null;
				for (int i = 0; i < nodeList.getLength(); i++) {
					node = nodeList.item(i);				
					attributes = node.getAttributes();
					
					 idTemp = attributes.getNamedItem("id").getNodeValue();
					 String editorImage =attributes.getNamedItem("editorImage").getNodeValue();
					 String errorImage = attributes.getNamedItem("errorImage").getNodeValue();
					 String paletteImage = attributes.getNamedItem("paletteImage").getNodeValue();
					 if (idTemp.equalsIgnoreCase(customerComponentId)){
						  if (model instanceof ServiceEngineModel) {		
							return new SEConcreteFigure(editorImage, errorImage, paletteImage);
						}else{
							return new BCConcreteFigure(idTemp, editorImage, errorImage, paletteImage);
						}
					 }
				}
			}
		}
		//create the figure with the component model name.
		String [] strs = this.classNameModel.split("\\.");
		String figureName = "";
		if (strs[strs.length - 2].equals(SEPREFIXEMODEL)) {
			figureName = SEPACKAGE;
		} else {
			figureName = BCPACKAGE;
		}
		figureName += "." + strs[strs.length - 1] + "Figure";
		try {
			return (IFigure) CimeroClassLoader.newInstance(
					figureName, new Object[] {});
		} catch (Exception e) {
			CimeroLog.errorLog("the part figure hasn't been find", e);
			
			return null;
		}
	}

	/**
	 * method to create a figure.
	 * @return IFigure : the figure
	 */
	@Override
	protected final IFigure createFigure() {
		IFigure fig = getCimeroFigure();
		if (fig instanceof BCCimeroFigure) {
			//set the name if it's a binding component
			String name = ((GeneriqueCimeroModel) getModel()).getName();
			((BCCimeroFigure) fig).setName(name);
		}
		return fig;
	}

	/**
	 * method to create policies.
	 */
	@Override
	protected final void createEditPolicies() {
		//policy to delete component
		installEditPolicy(EditPolicy.COMPONENT_ROLE,
				new NodeRemoveEditPolicy());
		//policy to avoid user to connect components
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
				new NodeGraphicalNodeEditPolicy());
		//policy to get the selection
		installEditPolicy(EditPolicy.SELECTION_FEEDBACK_ROLE,
				new NodeSelectionEditPolicy(getViewer()));
	}

	/**
	 * method called when properties change.
	 * @param evt : the evenement occured
	 */
	public final void propertyChange(final PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals(GeneriqueCimeroModel.LOCATION)) {
			//refresh when change property
			refreshVisuals();
		} else if (evt.getPropertyName().equals(GeneriqueCimeroModel.NAME)) {
			//change the name on the figure (label)
			String name = ((GeneriqueCimeroModel) getModel()).getName();
			((GeneriqueCimeroFigure) getFigure()).setName(name);
			//refresh when change name
			refreshVisuals();
		} else if (evt.getPropertyName().equals(GeneriqueCimeroModel.INPUTS)) {
			//refresh when change target connection
			refreshTargetConnections();
			//refresh
			refreshVisuals();
		} else if (evt.getPropertyName().equals(GeneriqueCimeroModel.OUTPUTS)) {
			//refresh when change source connection
			refreshSourceConnections();
			//refresh
			refreshVisuals();
		}
	}

	/**
	 * This function is used to return the ConnectionAnchor of the source
	 * element when the user selected the object.
	 * @param connection : the connection
	 * @return ConnectionAnchor : the anchor
	 */
	public ConnectionAnchor getSourceConnectionAnchor(
			final ConnectionEditPart connection) {
		ConnectionAnchor cA =
			((GeneriqueCimeroFigure) getFigure()).getConnectionAnchor("output");
		return cA;
	}

	/**
	 * This function is used to return the ConnectionAnchor of the target
	 * element when the user selected the object.
	 * @param connection : the connection
	 * @return ConnectionAnchor : the anchor
	 */
	public final ConnectionAnchor getTargetConnectionAnchor(
			final ConnectionEditPart connection) {
		return ((GeneriqueCimeroFigure) getFigure())
					.getConnectionAnchor("input");
	}

	/**
	 * This function is used to return the ConnectionAnchor of the source
	 * element when the user selected the object.
	 * @param request The request parameter
	 * @return ConnectionAnchor : the anchor
	 */
	public final ConnectionAnchor getSourceConnectionAnchor(
				final Request request) {
		Point pt = new Point(((DropRequest) request).getLocation());
		ConnectionAnchor source = ((GeneriqueCimeroFigure) getFigure())
					.getSourceConnectionAnchorAt(pt);
		return source;

	}

	/**
	 * This function is used to return the ConnectionAnchor of the target
	 * element when the user selected the object.
	 * @param request The request parameter
	 * @return ConnectionAnchor : the anchor
	 */
	public final ConnectionAnchor getTargetConnectionAnchor(
				final Request request) {
		Point pt = new Point(((DropRequest) request).getLocation());
		return ((GeneriqueCimeroFigure) getFigure())
					.getTargetConnectionAnchorAt(pt);
	}

	/**
	 * This function is used to draw the figure of this part.
	 */
	public final void refreshVisuals() {
		GeneriqueCimeroModel node = (GeneriqueCimeroModel) this.getModel();
		Point loc = node.getLocation();
		//if the figure is a BCFigure.
		if (getFigure() instanceof INOUTFigure) {
			//we refresh the figure
			((INOUTFigure) getFigure()).refreshPort((
					(BindingComponentModel) getModel()).isInput(), (
							(BindingComponentModel) getModel()).isOutput());
		}
		//call to the isValid function
		if (isBShowErrors()) {
			boolean bValide = false;
			//if one or more validation was done
			bValide = ((GeneriqueCimeroModel) getModel()).isValide(false);
			//
			if (!bValide) {
				//if an error was found
				((GeneriqueCimeroFigure) getFigure()).setValideDraw(false);
			} else {
				//no error
				((GeneriqueCimeroFigure) getFigure()).setValideDraw(true);
			}
		} else {
			//else, we put the normal mode
			((GeneriqueCimeroFigure) getFigure()).setValideDraw(true);
		}
		//we retreive the size of the figure
		Dimension size = ((GeneriqueCimeroFigure) getFigure())
					.getFigureDimension();
		//we create a rectangle with this dimesion
		Rectangle rectangle = new Rectangle(loc, size);
		//Eclipse draw this figure in this rectange
		((GraphicalEditPart) getParent())
					.setLayoutConstraint(this, getFigure(), rectangle);
	}

	/**
	 * override activate to register with the model for property changes.
	 */
	public final void activate() {
		if (isActive()) {
			return;
		}
		super.activate();
		((GeneriqueCimeroModel) getModel()).addPropertyChangeListener(this);
		//add the diagram reference in the generiqueModel
		((GeneriqueCimeroModel) getModel()).setDiagram(diagram);
	}

	/**
	 * override deactivate to deregister with the model.
	 */
	public final void deactivate() {
		if (!isActive()) {
			return;
		}
		super.deactivate();
		((GeneriqueCimeroModel) getModel()).removePropertyChangeListener(this);
	}

	/**
	 * method to get outgoing connections of a component.
	 * used to draw the connection beetween two object.
	 * @return List : outgoing connections
	 */
	protected final List getModelSourceConnections() {
		return ((GeneriqueCimeroModel)
				this.getModel()).getOutgoingConnections();
	}

	/**
	 * method to get incoming connections of a component.
	 * used to draw the connection beetween two object.
	 * @return List : incoming connections
	 */
	protected final List getModelTargetConnections() {
		return ((GeneriqueCimeroModel)
				this.getModel()).getIncomingConnections();
	}

	/**
	 * get the boolean which shows or not errors.
	 * @return Returns the bShowErrors.
	 */
	public final boolean isBShowErrors() {
		return bShowErrors;
	}

	/**
	 * set the boolean which shows or not errors.
	 * @param showErrors The bShowErrors to set
	 */
	public final void setBShowErrors(final boolean showErrors) {
		bShowErrors = showErrors;
	}
}
