package com.bull.cimero.pluginEditor.editors.policies;

import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.ConnectionEditPolicy;
import org.eclipse.gef.requests.GroupRequest;
import com.bull.cimero.pluginEditor.editors.commands.ConnectionDeleteCommand;
import com.bull.cimero.pluginEditor.editors.model.Connection;
import com.bull.cimero.pluginEditor.editors.parts.ConnectionPart;

/**
 * policy to remove connections.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class ConnectionRemoveEditPolicy extends ConnectionEditPolicy {

	/**
	 * default constructor.
	 */
	public ConnectionRemoveEditPolicy() {
		super();
	}

	/**
	 * get the figure of the connection.
	 * @return PolylineConnection the figure
	 */
	protected final PolylineConnection getConnectionFigure() {
		return (PolylineConnection) ((GraphicalEditPart) getHost()).getFigure();
	}

	/**
	 * get the delete method.
	 * @param request : the user request
	 * @return Command the good command
	 * @see org.eclipse.gef.editpolicies.ConnectionEditPolicy
	 * #getDeleteCommand(org.eclipse.gef.requests.GroupRequest)
	 */
	@Override
	protected final Command getDeleteCommand(final GroupRequest request) {
		ConnectionDeleteCommand connectionDeleteCommand =
			new ConnectionDeleteCommand();
		ConnectionPart connPart = (ConnectionPart) getHost();
		Connection conn = (Connection) connPart.getModel();
		//pass the the target
		connectionDeleteCommand.setTarget(conn.getTarget());
		//pass the source
		connectionDeleteCommand.setSource(conn.getSource());
		//pass the current connection
		connectionDeleteCommand.setConnection(conn);
		//return the command
	    return connectionDeleteCommand;
	}

	/**
	 * get the adequat method.
	 * @param request : the user request
	 * @return Command the good command
	 */
	@Override
	public final Command getCommand(final Request request) {
		//fonction of the request
		if (REQ_DELETE.equals(request.getType())) {
			return getDeleteCommand((GroupRequest) request);
		}
		return null;
	}
}
