package com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;

/**
 * This is the servicemix interface factory. This interface contains
 * a default constructor and the create function.
 * This function creates the servicemix model with a cimero component.
 * @author Cedric Mouilleron
 * @author Pierre Notel
 *@author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public interface GenericServiceMixFactory {

	/**
	 * This function will be used to create in a xml configuration
	 * file, an activation spec that represents an element of the cimero
	 * model.
	 * @param elt : the element using to create this activation spec
	 * @param isForce : true to not launch the validation element.
	 * @return the activation spec.
	 * @throws Exception : if the is a error during generation
	 */
	ActivationSpecClass createXMLModel(
			GeneriqueCimeroModel elt, Boolean isForce) throws Exception;

	/**
	 * @return the namespace using by this component in ServiceMix.
	 */
	NameSpaceClass getNamespaceComponent();
}
