package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.factory.lWFactory.SE;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Routeur;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.LWBeanClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.PropertyClass;

/**
 * This class will be use to add in the ServiceMix model, an activation
 * spec that contain a routeur.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class RouteurFactory extends SELWFactory {

	/**This variable contain the class of servicemix using
	 * to create a drools routeur.*/
	public static final String CLASS
		= "org.apache.servicemix.components.drools.DroolsComponent";
	/**The name of the namespace using by the routeur.*/
	public static final String NAMESPACENAME = "drools";
	/**The value of this namespace.*/
	public static final String NAMESPACEVALUE = "http://servicemix.org/drools";
	/**The name of the only one method of the routeur.*/
	private static final String PROPERTYNAME = "ruleBaseResource";
	/**The value of the propertyName.*/
	public static final String FILENAME = "droolsFile.xml";
	/**the prfixe using by the jar generation.*/
	private String prefixeJar = "";

	/**
	 * default constructor.
	 * @param ref : the reference of the current namespace.
	 */
	public RouteurFactory(final NameSpaceClass ref) {
		super(ref);
	}

	/**
	 * this function will create an activation spec that represents
	 * a drools routeur in serviceMix.
	 * @param elt : the element using to create this activation spec.
	 * @param force : false if the validation must to be do
	 * @return the activation spec create.
	 * @throws Exception : if the is a error during generation
	 */
	public final ActivationSpecClass createXMLModel(
			final GeneriqueCimeroModel elt,
			final Boolean force) throws Exception {
		ActivationSpecClass myActi = null;
		//if not null and valide and instance of router
		if ((elt != null) && (elt instanceof Routeur)) {
			if ((elt.isValide(false)) || (force)) {
				Routeur monRout = (Routeur) elt;
				//create the activationSpec
				myActi = new ActivationSpecClass();
				myActi.setNameSpaceRef(getNamespaceRef());
				//set the name
				myActi.setComponentName(monRout.getName());
				//set the service
				myActi.setServiceName(monRout.getName(),
						new NameSpaceClass(NAMESPACENAME , NAMESPACEVALUE));
				//create the components
				ComponentClass compo = createComponent(monRout);
				myActi.setComponent(compo);
				return myActi;
			}
		}
		throw new Exception("The generique cimero object is not valide");
	}

	/**
	 * This function creates the component of the servicemix drools routeur.
	 * @param elt : the cimero model element
	 * @return the instance of the component class.
	 */
	private ComponentClass createComponent(final Routeur elt) {
		//create the componentclass instance
		ComponentClass monCompo = new ComponentClass(getNamespaceRef());
		//add the bean class.
		LWBeanClass monB = createBeanRouteur(elt);
		monCompo.setMonBean(monB);
		return monCompo;
	}

	/**
	 * This function creates the bean using by the routeur in the
	 * serviceMix model.
	 * @param elt : the cimero element
	 * @return the bean routeur.
	 */
	private LWBeanClass createBeanRouteur(final Routeur elt) {
		//create the bean
		LWBeanClass beanRout = new LWBeanClass(CLASS);
		//add the property to this bean
		beanRout.addProperty(createPropertyRouteur(elt.getName()) , "");
		return beanRout;
	}

	/**
	 * This function creates the only one bean of the servicemix
	 * drools routeur.
	 * @param routeurName : the name of this routeur
	 * @return the property.
	 */
	private PropertyClass createPropertyRouteur(final String routeurName) {
		PropertyClass myProp = new PropertyClass(PROPERTYNAME);
		if (prefixeJar.compareTo("") == 0) {
			myProp.addValue(routeurName + FILENAME);
		} else {
			myProp.addValue(prefixeJar + routeurName + ".xml");
		}
		return myProp;
	}

	/**
	 * @param prefixeJarP the prefixeJar to set.
	 */
	public final void setPrefixeJar(final String prefixeJarP) {
		this.prefixeJar = prefixeJarP;
	}
}
