package com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.BC;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.TCPBC;
import com.bull.cimero.pluginEditor.editors.model.BCModel.TCPModelHelper;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.GeneriqueLWfactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.StandardComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardJBIClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardTCPClass;
/**
 * @author Nicola Buso (nicola.buso@eng.it)
 *  
 */
public class TCPBCFactory extends BCStdFactory {

	/**The tcp namespace name.*/
	public static final String NAMESPACENAME = "tcp";
	/**The tcp namespace value.*/
	public static final String NAMESPACEVALUE =
		"http://servicemix.apache.org/tcp/1.0";

	/**default in-out mep mode url.*/
	private static final String DEFINOUTMEP =
		"http://www.w3.org/2004/08/wsdl/in-out";
	/**default in-only mep mode url.*/
	private static final String DEFINMEP =
		"http://www.w3.org/2004/08/wsdl/in-only";
	
	private static final String DEFOUTMEP = 
		"http://www.w3.org/2004/08/wsdl/out-only";

	
	
	public TCPBCFactory(NameSpaceClass nms) {
		super(nms, true);
	}
	
	public TCPBCFactory(NameSpaceClass nms, Boolean mepMode) {
		super(nms, mepMode);
	}

	@Override
	public String getNamespaceName() {
		// TODO Auto-generated method stub
		return NAMESPACENAME;
	}

	@Override
	public String getNamespaceValue() {
		// TODO Auto-generated method stub
		return NAMESPACEVALUE;
	}

	public ActivationSpecClass createXMLModel(GeneriqueCimeroModel elt,
			Boolean isForce) throws Exception {
		
		if (getNamespaceRef() == null) {
			throw new Exception("Namespaceclass cannot be null");
		}
		ActivationSpecClass myActi = null;
		//if not null and valide and instance of mail model
		if ((elt != null) && (elt instanceof TCPBC)) {
			if ((elt.isValide(false)) || (isForce)) {
				TCPBC myTCPBC = (TCPBC) elt;
				//create the activationSpec
				myActi = new ActivationSpecClass();
				myActi.setNameSpaceRef(getNamespaceRef());
				myActi.setComponentName(myTCPBC.getName());
				myActi.setServiceName(myTCPBC.getName(),
					new NameSpaceClass(GeneriqueLWfactory.NAMESPACENAME ,
							GeneriqueLWfactory.NAMESPACEVALUE));
				//we test if this element is in in mep mode or out mep mode.
				if (isInMepMode()) {
					//set the destination service
					setDestinationService(myActi , myTCPBC);
				}
				//create the components
				ComponentClass compo = createComponent(myTCPBC);
				myActi.setComponent(compo);
				return myActi;
			}
		 } else {
			throw new Exception("The generique cimero object is not valide");
		}
		return null;
	}

	public StandardJBIClass createJBIElement(GeneriqueCimeroModel elt)
			throws Exception {
		
		if (elt instanceof TCPBC) {
			TCPBC myTCPBC = (TCPBC) elt;
			
			String mepURI = myTCPBC.getDefaultMep();
			if (TCPModelHelper.IN_OUT == mepURI) {
				mepURI = DEFINOUTMEP;
			}
			if (TCPModelHelper.OUT_ONLY == mepURI) {
				mepURI = DEFOUTMEP;
			}
			if (TCPModelHelper.IN_ONLY == mepURI) {
				mepURI = DEFINMEP;
			}
			
			GeneriqueCimeroModel destElt = getDestination(elt);
			if (destElt != null){
				NameSpaceClass destinationNms = getNamespace(destElt);
				StandardTCPClass myTCP = new StandardTCPClass(
							myTCPBC.getName()
							, getNamespace(myTCPBC)
							, mepURI
							, new NameSpaceClass(getNamespaceName() ,
												getNamespaceValue())
							, destElt.getName()
							, destinationNms
							, myTCPBC.getName()
							, myTCPBC);
				myTCP.setCreatedByPipeline(myTCPBC.isCreatedByPipeline());
				return myTCP;			
			}else {
				StandardTCPClass myTCP = new StandardTCPClass(
						myTCPBC.getName()
						, getNamespace(myTCPBC)
						, mepURI
						, new NameSpaceClass(getNamespaceName() ,
											getNamespaceValue())
						, null
						, null
						, myTCPBC.getName()
						, myTCPBC);
				myTCP.setCreatedByPipeline(myTCPBC.isCreatedByPipeline());
				return myTCP;
			}
		}
		return null;
	}

	/**
	 * creates the servicemix WS component.
	 * @param myTCPBC : the cimero model element
	 * @return the instance of the component class.
	 * @throws Exception : if the generation return an error
	 */
	private ComponentClass createComponent(final TCPBC myTCPBC)
		throws Exception {
		//create the componentclass instance
		ComponentClass monCompo = new ComponentClass(getNamespaceRef());
		//add the bean class.
		StandardComponentClass monS = null;
		monS = createStandardComponentMEP(myTCPBC);
		monCompo.setMonComponentStandard(monS);
		return monCompo;
	}

	/**
	 * creates the standard components.
	 * @param monWS : the cimero WS
	 * @return the standard component class.
	 * @throws Exception : if the generation return an error.
	 */
	private StandardComponentClass createStandardComponentMEP(
			final TCPBC myTCPBC) throws Exception {
		StandardComponentClass monStd = new StandardComponentClass(
				new NameSpaceClass(getNamespaceName() , getNamespaceValue()));
		monStd.setMyJBICompo(createJBIElement(myTCPBC));
		return monStd;
	}


}
