package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.factory.standardFactory.SE;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.GeneriqueLWfactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.StandardComponentClass;

/**
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 *@author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public abstract class StandardEIPFactory extends SEStdFactory {

	/**The http namespace name.*/
	public static final String NAMESPACENAME = "eip";
	/**The http namespace value.*/
	public static final String NAMESPACEVALUE =
		"http://servicemix.apache.org/eip/1.0";


	/**
	 * default constructor.
	 * @param nms : the namespace using by this factory.
	 */
	public StandardEIPFactory(final NameSpaceClass nms) {
		super(nms);
	}

	/**
	 * @see com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.GeneriqueStandardFactory
	 * #createXMLJBIElement(com.bull.cimero.pluginEditor.editors
	 * .model.GeneriqueCimeroModel, com.bull.cimero.pluginEditor
	 * .generator.serviceMixGenerator.model.global.NameSpaceClass)
	 * @param elt : the generique cimero element.
	 * @param isForce : true if the validation not to be do.
	 * @throws Exception if there is a problem during generation.
	 * @return the activation spec class that contain this component.
	 */
	public final ActivationSpecClass createXMLModel(
			final GeneriqueCimeroModel elt,
			final Boolean isForce) throws Exception {
		if (getNamespaceRef() == null) {
			throw new Exception("namespaceclass cannot be null");
		}
		ActivationSpecClass myActi = null;
		//if not null and valide
		if (elt != null) {
			if ((elt.isValide(false)) || (isForce)) {
				//create the activationSpec
				myActi = new ActivationSpecClass();
				myActi.setNameSpaceRef(getNamespaceRef());
				myActi.setComponentName(elt.getName());
				myActi.setServiceName(elt.getName(),
					new NameSpaceClass(GeneriqueLWfactory.NAMESPACENAME ,
							GeneriqueLWfactory.NAMESPACEVALUE));
				//create the components
				ComponentClass compo = createComponent(elt);
				myActi.setComponent(compo);
				return myActi;
			}
		 } else {
			throw new Exception("The generique cimero object is not valide");
		}
		return null;
	}

	/**
	 * create the component of the xml generation.
	 * @param elt :  the EIP model.
	 * @return the generated component instance class
	 * @throws Exception : if the generation return an error.
	 */
	private ComponentClass createComponent(
			final GeneriqueCimeroModel elt)
			throws Exception {
//		create the componentclass instance
		ComponentClass monCompo = new ComponentClass(getNamespaceRef());
		//add the bean class.
		StandardComponentClass monS = null;
		monS = createStandardComponentMEP(elt);
		monCompo.setMonComponentStandard(monS);
		return monCompo;
	}

	/**
	 * creates the standard components.
	 * @param elt : the cimero Wiretap model
	 * @return the standard component class.
	 * @throws Exception
	 * @throws Exception : if the generation return an error.
	 */
	private StandardComponentClass createStandardComponentMEP(
			final GeneriqueCimeroModel elt) throws Exception {
		StandardComponentClass monStd = new StandardComponentClass(
				new NameSpaceClass(getNamespaceName() , getNamespaceValue()));
		monStd.setMyJBICompo(createJBIElement(elt));
		return monStd;
	}

	/**
	 * @see com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.GeneriqueStandardFactory#getNamespaceName()
	 * @return the default namespace.
	 */
	public final String getNamespaceName() {
		return NAMESPACENAME;
	}

	/**
	 * @see com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.GeneriqueStandardFactory#getNamespaceValue()
	 * @return the default namespace value.
	 */
	public final String getNamespaceValue() {
		return NAMESPACEVALUE;
	}

	/**
	 * @return the namespace using by this servicemix component.
	 */
	public final NameSpaceClass getNamespaceComponent() {
		return new NameSpaceClass(getNamespaceName(), getNamespaceValue());
	}
}
