package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.factory.standardFactory.SE;

import com.bull.cimero.pluginEditor.editors.model.Connection;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.SEModel.WireTap;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.GeneriqueLWfactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardEIPWireTap;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardJBIClass;

/**
 * Factory to create a standard Eip WireTap component.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class WireTapFactory extends StandardEIPFactory {

	/**
	 * default constructor.
	 * @param nms : the namespace using by this factory.
	 */
	public WireTapFactory(final NameSpaceClass nms) {
		super(nms);
	}

	/**
	 * @see com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.GeneriqueStandardFactory
	 * #createJBIElement(com.bull.cimero.pluginEditor.editors.model
	 * .GeneriqueCimeroModel)
	 * @param elt : the generique cimero element.
	 * @return the standard model of this wiretap eip.
	 * @throws Exception if the generation had a problem.
	 */
	public final StandardJBIClass createJBIElement(
			final GeneriqueCimeroModel elt)
			throws Exception {
		StandardEIPWireTap myWT = null;
		if (elt instanceof WireTap) {
			WireTap wireTapModel = (WireTap) elt;
			GeneriqueCimeroModel destElt =
					((Connection) wireTapModel.getConn(false)).getTarget();
			GeneriqueCimeroModel destCopy =
					((Connection) wireTapModel.getConn(true)).getTarget();
			if ((destElt != null) && (destCopy != null)) {
				NameSpaceClass destinationNms = getNamespace(destElt);
				NameSpaceClass destinationCopyNms = getNamespace(destCopy);
				myWT = new StandardEIPWireTap(wireTapModel.getName(),
						new NameSpaceClass(GeneriqueLWfactory.NAMESPACENAME,
											GeneriqueLWfactory.NAMESPACEVALUE),
						new NameSpaceClass(getNamespaceName(),
										  getNamespaceValue()),
						wireTapModel.getName(),
						destElt.getName(),
						destinationNms,
						destCopy.getName(),
						destinationCopyNms);
			} else {
					throw new Exception("The destination component of the "
						+ wireTapModel.getName() + " object doesn't exist");
			}
		} else {
			throw new Exception("the object : "
					+ elt.getName() + " hasn't a good type for this factory");
		}
		return myWT;
	}
}
