package com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global;

import java.util.ArrayList;
import java.util.List;

import com.bull.cimero.pluginEditor.editors.model.SEModel.Routeur.Rule;

/**
 * This class contains all the informations of a drools file for the
 * routeur of serviceMix.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class DroolsRouterModel {

	/**This list contain all the touter rules.*/
	private List < ServiceMixRule > lstRule = null;
	/**The prefixe of this drools file.*/
	private String prefixeName = "";

	/**
	 * The default constructor.
	 */
	public DroolsRouterModel() {
		lstRule = new ArrayList < ServiceMixRule >();
	}

	/**
	 * add to the rules list, a rule.
	 * @param name : the name of this rule.
	 * @param myRule : the rule to add.
	 */
	public final void addRule(final String name , final List < Rule > myRule) {
		lstRule.add(new ServiceMixRule(myRule , name));
	}

	/**
	 * @return the number of element of the rule list.
	 */
	public final int lstRuleSize() {
		return lstRule.size();
	}

	/**
	 * THis function return the rule of the lstRule which is at
	 * the specifiy position.
	 * @param position : the position of the rule.
	 * @return rule.
	 */
	public final ServiceMixRule getRuleElt(final int position) {
		if ((position < lstRuleSize()) && (position >= 0)) {
			return lstRule.get(position);
		} else {
			return null;
		}
	}

	/**
	 * This class represent a rule in the serviceMix model.
	 * It contain a name and a rule.
	 */
	public class ServiceMixRule {
		/**The rule.*/
		private List < Rule > myRule = null;
		/**The name of this class.*/
		private String name = "";

		/**
		 * The constructor with parameter.
		 * @param myRuleP : the rule.
		 * @param nameP : the name of this class.
		 */
		public ServiceMixRule(final List < Rule > myRuleP, final String nameP) {
			super();
			this.myRule = myRuleP;
			this.name = nameP;
		}
		/**
		 * @return Returns the myRule.
		 */
		public final  List < Rule > getMyRule() {
			return myRule;
		}
		/**
		 * @return Returns the name.
		 */
		public final String getName() {
			return name;
		}
	}

	/**
	 * @return the prefixeName
	 */
	public final String getPrefixeName() {
		if (prefixeName.compareTo("") != 0) {
			return prefixeName.split("\\.")[0];
		} else {
			return "";
		}
	}

	/**
	 * @param prefixeNameP the prefixeName to set
	 */
	public final void setPrefixeName(final String prefixeNameP) {
		this.prefixeName = prefixeNameP;
	}

}
