package com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global;

import java.util.ArrayList;
import java.util.List;

/**
 * This class represents the a bean in ServiceMix model.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 *@author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class LWBeanClass {
	/**The name of the class using by this bean.*/
	private String className = "";
	/**The optional xmlns.*/
	private String xmlnsValue = "";
	/**a boolean to determine if this bean reference a isProperty or
	 * a constructorClass.*/
	private boolean isProperty = false;
	/**The reference to the PropertyRef model.*/
	private List < PropertyRef > lstProp = null;
	/**The reference to the ConstructorClass model.*/
	private ConstructorClass myConstructor = null;


	//---------------------------------------------------
	//CONSTRUCTOR
	/**
	 * The constructor with the classname in parameter.
	 * @param classNameParam : the name of the class using by servicemix
	 * 		to implement this bean.
	 */
	public LWBeanClass(final String classNameParam) {
		lstProp = new ArrayList < PropertyRef >();
		this.className = classNameParam;
	}

	//-----------------------------------------------------------
	//SETTERS FUNCTION
	/**
	 * This function is used to set a propertyClass to this bean.
	 * @param myPropParam : the propertyclass model.
	 * @param xmlsParam : the optional xmls.
	 */
	public final void addProperty(final PropertyClass myPropParam,
			final String xmlsParam) {
			this.isProperty = true;
			this.lstProp.add(new PropertyRef(xmlsParam, myPropParam));
	}

	/**
	 * Set the name of this bean.
	 * @param classNameParam : the name of this bean.
	 */
	public final void setClassName(final String classNameParam) {
		this.className = classNameParam;
	}

	/**
	 * set the constructor class of this bean.
	 * @param constr : the constructor reference.
	 */
	public final void setConstructor(final ConstructorClass constr) {
		if (constr != null) {
			this.isProperty = false;
			this.lstProp.clear();
			this.myConstructor = constr;
		}
	}

	//-----------------------------------------------------------
	//GETTER FUNCTIONS
	/**
	 * @return the name of this bean.
	 */
	public final String getClassName() {
		return className;
	}

	/**
	 * @return the constructorClass of this bean.
	 */
	public final ConstructorClass getMyConstructor() {
		return myConstructor;
	}

	/**
	 * @return the isProperty of this bean.
	 */
	public final List getlstProp() {
		List < PropertyClass > maLst = new ArrayList < PropertyClass >();
		for (int i = 0; i < lstProp.size(); i++) {
			PropertyClass tmp = ((PropertyRef) lstProp.get(i)).getMyProperty();
			maLst.add(tmp);
		}
		return maLst;
	}

	/**
	 * @return true if the bean has a isProperty and false in the other
	 * case.
	 */
	public final boolean isProperty() {
		return isProperty;
	}


	/**
	 * @return Returns the xmlnsValue.
	 */
	public final String getXmlnsValue() {
		return xmlnsValue;
	}

	/**
	 * @param xmlnsValueP The xmlnsValue to set.
	 */
	public final void setXmlnsValue(final String xmlnsValueP) {
		this.xmlnsValue = xmlnsValueP;
	}


//	-----------------------------------------------------------
	//INNER CLASS
	/**
	 * This class reference a isProperty and the xmls using by this isProperty.
	 * Ths xmls is optional.
	 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
	 *
	 */
	private class PropertyRef {
		/**The name of the xmls.*/
		private String xmlsName = "";
		/**The reference to the propertyClass.*/
		private PropertyClass myProperty = null;

		/**
		 * This constructor with 2 parameters.
		 * @param xmlsNameParam : the optional xmls name.
		 * @param myPropertyParam : the reference to the isProperty.
		 */
		public PropertyRef(final String xmlsNameParam,
				final PropertyClass myPropertyParam) {
			super();
			this.xmlsName = xmlsNameParam;
			this.myProperty = myPropertyParam;
		}

		/**
		 * @return the propertyClass.
		 */
		public final PropertyClass getMyProperty() {
			return myProperty;
		}

		/**
		 * @return the name of the Xmls.
		 */
		public final String getXmlsName() {
			return xmlsName;
		}

	}






}
