package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.model.global.standardJBIClassImplementation;

import it.eng.utils.PropertyModel;

import java.util.ArrayList;
import java.util.List;

import com.bull.cimero.pluginEditor.editors.model.BCModel.FileBC;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.GeneriqueLWfactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;

/**
 * This class represents the standard JBI File component in the
 * ServiceMix model.
 * @author Daniela Butano
 *@author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class StandardFileClass extends StandardJBIClass implements IBindingComponentClass {

	/**the defaultMep value.*/
	private String defaultMepValue = "";
	public static final String MEP_IN = "INPUT";
	public static final String MEP_OUT = "OUTPUT";
	public FileBC fileBC = null;
	/**
	 * constructor with parameters.
	 * @param defMepValue : the mep value
	 * @param srcName : the service destination name
	 * @param serviceNamespaceParam : the namespace using by the service.
	 * @param jBINamespace : the namespace reference using by the service
	 * @param endPValue : the endpoint value
	 * @param providerRoleParam :
	 * 				the role of this component : PROVIDER or CONSUMER
	 * @param locURI : the location URI
	 * @param soap : true if it is a soap request
	 *
	 */
	public StandardFileClass(final String defMepValue,
			final String srcName,
			final NameSpaceClass serviceNamespaceParam,
			final NameSpaceClass jBINamespace,
			final String endPValue,
			final FileBC fileBC 
			) {
		super(jBINamespace);
		this.defaultMepValue = defMepValue;
		this.serviceNamespace = serviceNamespaceParam;
		this.serviceName = srcName;
		this.endpointValue = endPValue;
		this.fileBC = fileBC;
	}

	
	
	
	
	/**
	 * @return Returns the serviceName.
	 */
	public final String getServiceName() {
		return serviceName;
	}

	
	
	public String getDefaultMepValue() {
		return defaultMepValue;
	}

	public void setDefaultMepValue(String defaultMepValue) {
		this.defaultMepValue = defaultMepValue;
	}

	public String getXBean(){
		StringBuffer sb = new StringBuffer();
		if (defaultMepValue.equals(MEP_IN)) {
			sb.append("<file:poller service=\""
					+ GeneriqueLWfactory.NAMESPACENAME+ ":"
					+ this.endpointValue + "\"\n");
			sb.append("     endpoint=\"" + this.endpointValue + "\"\n");
			sb.append("     targetService=\"" + serviceNamespace.getNameSpaceName()
					+ ":" + this.serviceName + "\"\n");
			sb.append("     file=\"" + fileBC.getPropertyValue(FileBC.FILE) + "\"\n");
			sb.append("     deleteFile=\"").append(fileBC.isDeleteFile()).append("\"\n");
			sb.append("     period=\"").append(fileBC.getProperty(FileBC.PERIOD)).append("\"\n");
			sb.append(">");
			sb.append(getMarshallerXbeanConfig());
			sb.append("</file:poller>\n");
		} else {
			sb.append("<file:sender service=\""
					+ GeneriqueLWfactory.NAMESPACENAME+ ":"
					+ this.endpointValue + "\"\n");
			sb.append("     endpoint=\"" + this.endpointValue + "\"\n");
			sb.append("     directory=\"" + fileBC.getPropertyValue(FileBC.DIRECTORY) + "\"\n");
			sb.append("     append=\"" + fileBC.isAppendFile() + "\"\n");
			sb.append(">");
			sb.append(getMarshallerXbeanConfig());
			sb.append("</file:sender>\n");
		}
		return sb.toString();
	}
	
	private String getMarshallerXbeanConfig() {
		StringBuffer cfg = new StringBuffer(
			"<file:marshaler>"
				+"<sm:").append(fileBC.getFileMarshalerValue()).append(">");
//					+"<sm:fileName>"
//						+"<sm:xpathString xpath=\"concat($name, '.xml')\"/>"
//					+"</sm:fileName>"
					if (fileBC.getFileMarshalerValue().equalsIgnoreCase(FileBC.BINARY_FILE_MARSHALLER))
						cfg.append("<sm:contentType>").append(fileBC.getProperty(FileBC.BINARY_CONTENT_TYPE)).append("</sm:contentType>");
					cfg.append("</sm:").append(fileBC.getFileMarshalerValue()).append(">"
			+"</file:marshaler>");
		
		return cfg.toString();
	}
	
	@Override
	public List<PropertyModel> getPropertiesToStore() {
		
		List<PropertyModel> propertiesToStore = new ArrayList<PropertyModel>();
		StringBuffer sb = new StringBuffer();
		if (defaultMepValue.equals(MEP_IN)) {
			propertiesToStore.add(createPropertyModel("role"," poller "));
			propertiesToStore.add(createPropertyModel("file",(String)fileBC.getPropertyValue(FileBC.FILE)));
		}else{
			propertiesToStore.add(createPropertyModel("role"," sender "));
			propertiesToStore.add(createPropertyModel("directory",(String)fileBC.getPropertyValue(FileBC.DIRECTORY)));
		}
		
		return propertiesToStore;
	}
}
