package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.model.global.standardJBIClassImplementation;

import it.eng.utils.PropertyModel;

import java.util.ArrayList;
import java.util.List;

import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;

/**
 * This class represents the standard JBI HTTP component in the
 * ServiceMix model.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 *@author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class StandardHttpClass extends StandardJBIClass implements IBindingComponentClass {

	/**The provider role.*/
	public static final String PROVIDER = "provider";
	/**The consumer role.*/
	public static final String CONSUMER = "consumer";

	/**the defaultMep value.*/
	private String defaultMepValue = "";
	/**the role of this provider.*/
	private String providerRole = "";
	/**the location URI.*/
	private String locationURI = "";
	/**a boolean, to determine if it is a soap request.*/
	private boolean isSoap = false;
	
	private boolean sslEnabled = false;
	private String sslKeyStore = null;
	private String sslkeystorePassword = null;
	private String sslkeyStoreType = null;
	private String sslTrustStore = null;
	private String sslTrustStorePassword = null;
	private String sslTrustStoreType = null;
	private boolean wantClientAuth = false;
	private boolean needClientAuth = false;
	
	private boolean createdByPipeline = false;
	private String relativeWSDLResource = null;
	private String soapAction = "";
	private String soapVersion = "";

	/**
	 * constructor with parameters.
	 * @param defMepValue : the mep value
	 * @param srcName : the service destination name
	 * @param serviceNamespaceParam : the namespace using by the service.
	 * @param jBINamespace : the namespace reference using by the service
	 * @param endPValue : the endpoint value
	 * @param providerRoleParam :
	 * 				the role of this component : PROVIDER or CONSUMER
	 * @param locURI : the location URI
	 * @param soap : true if it is a soap request
	 *
	 */
	public StandardHttpClass(final String defMepValue,
			final String srcName,
			final NameSpaceClass serviceNamespaceParam,
			final NameSpaceClass jBINamespace,
			final String endPValue,
			final String providerRoleParam,
			final String locURI,
			final boolean soap,
			final String soapV,
			final String soapA) {
		super(jBINamespace);
		this.defaultMepValue = defMepValue;
		this.serviceNamespace = serviceNamespaceParam;
		this.serviceName = srcName;
		this.endpointValue = endPValue;
		this.providerRole = providerRoleParam;
		this.locationURI = locURI;
		this.isSoap = soap;
		this.soapVersion = soapV;
		this.soapAction = soapA;
	}

	//----------------------------------------------------
	//GETTER FUNCTIONS
	/**
	 * @return Returns the defaultMepValue.
	 */
	public final String getDefaultMepValue() {
		return defaultMepValue;
	}
	
	/**
	 * @return Returns the isSoap.
	 */
	public final boolean isSoap() {
		return isSoap;
	}
	/**
	 * @return Returns the locationURI.
	 */
	public final String getLocationURI() {
		return locationURI;
	}
	/**
	 * @return Returns the providerRole.
	 */
	public final String getProviderRole() {
		return providerRole;
	}
	/**
	 * @return Returns the serviceName.
	 */
	public final String getServiceName() {
		return serviceName;
	}

	/**
	 * @return Returns the serviceNamespace.
	 */
	

	public String getRelativeWSDLResource() {
		return relativeWSDLResource;
	}

	public void setRelativeWSDLResource(String relativeWSDLResource) {
		this.relativeWSDLResource = relativeWSDLResource;
	}

	public boolean isNeedClientAuth() {
		return needClientAuth;
	}

	public void setNeedClientAuth(boolean needClientAuth) {
		this.needClientAuth = needClientAuth;
	}

	public boolean isSslEnabled() {
		return sslEnabled;
	}

	public void setSslEnabled(boolean sslEnabled) {
		this.sslEnabled = sslEnabled;
	}

	public String getSslKeyStore() {
		return sslKeyStore;
	}

	public void setSslKeyStore(String sslKeyStore) {
		this.sslKeyStore = sslKeyStore;
	}

	public String getSslkeystorePassword() {
		return sslkeystorePassword;
	}

	public void setSslkeystorePassword(String sslkeystorePassword) {
		this.sslkeystorePassword = sslkeystorePassword;
	}

	public String getSslTrustStore() {
		return sslTrustStore;
	}

	public void setSslTrustStore(String sslTrustStore) {
		this.sslTrustStore = sslTrustStore;
	}

	public String getSslTrustStorePassword() {
		return sslTrustStorePassword;
	}

	public void setSslTrustStorePassword(String sslTrustStorePassword) {
		this.sslTrustStorePassword = sslTrustStorePassword;
	}

	public boolean isWantClientAuth() {
		return wantClientAuth;
	}

	public void setWantClientAuth(boolean wantClientAuth) {
		this.wantClientAuth = wantClientAuth;
	}

	@Override
	public List<PropertyModel> getPropertiesToStore() {
		
		List<PropertyModel> propertiesToStore = new ArrayList<PropertyModel>();
		propertiesToStore.add(createPropertyModel("locationURI",locationURI));
		propertiesToStore.add(createPropertyModel("isSoap",String.valueOf(isSoap)));
		propertiesToStore.add(createPropertyModel("sslEnabled",String.valueOf(sslEnabled)));
		propertiesToStore.add(createPropertyModel("relativeWSDLResource",relativeWSDLResource));
		return propertiesToStore;
	}

	public boolean isCreatedByPipeline() {
		return createdByPipeline;
	}

	public void setCreatedByPipeline(boolean createdByPipeline) {
		this.createdByPipeline = createdByPipeline;
	}
	

	public String getSslkeyStoreType() {
		return sslkeyStoreType;
	}

	public void setSslkeyStoreType(String sslkeystoreType) {
		this.sslkeyStoreType = sslkeystoreType;
	}

	public String getSslTrustStoreType() {
		return sslTrustStoreType;
	}

	public String getSoapAction() {
		return soapAction;
	}

	public String getSoapVersion() {
		return soapVersion;
	}

	public void setSslTrustStoreType(String sslTrustStoreType) {
		this.sslTrustStoreType = sslTrustStoreType;
	}

	
}
