package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.model.global.standardJBIClassImplementation;

import it.eng.utils.PropertyModel;

import java.util.ArrayList;
import java.util.List;

import com.bull.cimero.pluginEditor.editors.model.BCModel.FileBC;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;

/**
 * The standard JMS Class.
 * @author Cedric Mouilleron
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class StandardJMSClass extends StandardJBIClass implements IBindingComponentClass {

	/**The provider role.*/
	public static final String PROVIDER = "provider";
	/**The consumer role.*/
	public static final String CONSUMER = "consumer";

	/**the defaultMep value.*/
	private String defaultMepValue = "";
	/**the role.*/
	private String role = "";
	/**The target service name.*/
	private String targetServiceName = "";
	/**The target service namespace.*/
	private NameSpaceClass targetServiceNms = null;
	/**The destination style.*/
	private String destinationStyle = "";
	/**The JMS provider destination.*/
	private String jMSProviderDestinationName = "";
	/**The jndi connection factory name.*/
	private String jndiConnectionFactoryName = "";

	/**
	 * constructor.
	 * @param jbiNamespace : jbi namespace
	 * @param serviceNameP : service name
	 * @param serviceNamespaceP : service namespace
	 * @param endpointValueP : endpoint value
	 * @param targetServiceNameP : target name
	 * @param targetServiceNmsP : target namespace
	 * @param roleP : role (provider, consumer)
	 * @param defaultMepValueP : mep value
	 * @param destinationStyleP : jms style (queue, topic)
	 * @param providerDestinationNameP : the provider name
	 * @param jndiConnectionFactoryNameP : the jndi connection factory name.
	 */
	public StandardJMSClass(final NameSpaceClass jbiNamespace,
			final String serviceNameP,
			final NameSpaceClass serviceNamespaceP,
			final String endpointValueP,
			final String targetServiceNameP,
			final NameSpaceClass targetServiceNmsP,
			final String roleP,
			final String defaultMepValueP,
			final String destinationStyleP,
			final String providerDestinationNameP,
			final String jndiConnectionFactoryNameP) {
		super(jbiNamespace);
		this.serviceName = serviceNameP;
		this.serviceNamespace = serviceNamespaceP;
		this.endpointValue = endpointValueP;
		this.targetServiceName = targetServiceNameP;
		this.targetServiceNms = targetServiceNmsP;
		this.role = roleP;
		this.defaultMepValue = defaultMepValueP;
		this.destinationStyle = destinationStyleP;
		this.jMSProviderDestinationName = providerDestinationNameP;
		this.jndiConnectionFactoryName = jndiConnectionFactoryNameP;
	}

	/**
	 * @return the defaultMepValue
	 */
	public final String getDefaultMepValue() {
		return defaultMepValue;
	}

	/**
	 * @return the destinationStyle
	 */
	public final String getDestinationStyle() {
		return destinationStyle;
	}

	

	/**
	 * @return the jMSProviderDestinationName
	 */
	public final String getJMSProviderDestinationName() {
		return jMSProviderDestinationName;
	}

	/**
	 * @return the jndiConnectionFactoryName
	 */
	public final String getJndiConnectionFactoryName() {
		return jndiConnectionFactoryName;
	}

	/**
	 * @return the role
	 */
	public final String getRole() {
		return role;
	}

	/**
	 * @return the serviceName
	 */
	public final String getServiceName() {
		return serviceName;
	}

	

	/**
	 * @return the targetServiceName
	 */
	public final String getTargetServiceName() {
		return targetServiceName;
	}

	/**
	 * @return the targetServiceNms
	 */
	public final NameSpaceClass getTargetServiceNms() {
		return targetServiceNms;
	}

	@Override
	public List<PropertyModel> getPropertiesToStore() {
		
		List<PropertyModel> propertiesToStore = new ArrayList<PropertyModel>();
		
		
		return propertiesToStore;
	}


}
