
package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.model.global.standardJBIClassImplementation;

import it.eng.utils.PropertyModel;

import java.util.ArrayList;
import java.util.List;

import com.bull.cimero.pluginEditor.editors.model.BCModel.TCPBC;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.GeneriqueLWfactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;

/**
 * This class represents the standard JBI TCP component in the
 * ServiceMix model.
 *
 @author Nicola Buso (nicola.buso@eng.it)
 */
public class StandardTCPClass extends StandardJBIClass implements IBindingComponentClass {

	/**The provider role.*/
	public static final String PROVIDER = "provider";
	/**The consumer role.*/
	public static final String CONSUMER = "consumer";
	/**The target service name.*/
	private String targetServiceName = "";
	/**The target service namespace.*/
	private NameSpaceClass targetServiceNms = null;
	/**Default mep URI string*/
	private String defaultMep = "";
	private boolean createdByPipeline = false;
	
	public boolean isCreatedByPipeline() {
		return createdByPipeline;
	}

	public void setCreatedByPipeline(boolean createdByPipeline) {
		this.createdByPipeline = createdByPipeline;
	}
	public TCPBC tcpBC = null;
	/**
	 * constructor with parameters.
	 * @param defMepValue : the mep value
	 * @param srvDestName : the service destination name
	 * @param serviceNamespaceParam : the namespace using by the service.
	 * @param jBINamespace : the namespace reference using by the service
	 * @param endPValue : the endpoint value
	 * @param providerRoleParam :
	 * 				the role of this component : PROVIDER or CONSUMER
	 * @param locURI : the location URI
	 * @param soap : true if it is a soap request
	 *
	 */
	public StandardTCPClass(
			final String srvName,
			final NameSpaceClass serviceNamespaceParam,
			final String defaultMep,
			final NameSpaceClass jBINamespace,
			final String srvDestName,
			final NameSpaceClass serviceDestNamespaceParam,
			final String endPValue,
			final TCPBC tcpBc 
			) {
		super(jBINamespace);
	
		this.serviceNamespace = serviceNamespaceParam;
		this.serviceName = srvName;
		this.defaultMep = defaultMep;
		this.targetServiceName = srvDestName;
		this.targetServiceNms = serviceDestNamespaceParam;
		this.endpointValue = endPValue;
		this.tcpBC = tcpBc;
	}

	
	
	
	/**
	 * @return Returns the serviceName.
	 */
	public final String getServiceName() {
		return serviceName;
	}

	

	public String getRole() {
		return tcpBC.isOutput()? CONSUMER : PROVIDER;
	}
	
	public String getServiceInNameSpace() {
		return new StringBuffer(GeneriqueLWfactory.NAMESPACENAME+":").append(endpointValue).toString();
	}

	public TCPBC getTCPBC() {
		return tcpBC;
	}


	public String getTargetServiceName() {
		return targetServiceName;
	}


	public NameSpaceClass getTargetServiceNms() {
		return targetServiceNms;
	}


	public String getDefaultMep() {
		return defaultMep;
	}

	@Override
	public List<PropertyModel> getPropertiesToStore() {
		
		List<PropertyModel> propertiesToStore = new ArrayList<PropertyModel>();
		
		
		return propertiesToStore;
	}
}
