package com.bull.cimero.pluginEditor.popupMenu;

import java.beans.XMLEncoder;
import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.List;
import java.util.Set;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import com.bull.cimero.pluginEditor.editors.CimeroRegistry;
import com.bull.cimero.pluginEditor.editors.DiagramEditor;
import com.bull.cimero.pluginEditor.editors.model.Connection;
import com.bull.cimero.pluginEditor.editors.model.Diagram;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Routeur;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Routeur.Rule;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class ExportInXMLAction implements IObjectActionDelegate {

	/** the selected file. */
	private IFile fileSelected = null;

	/**
	 * Constructor for GenerateJarFileAction.
	 */
	public ExportInXMLAction() {
		super();
	}

	/**
	 * the getter fileselected function.
	 * @return IFile
	 */
	public final IFile getFileSelected() {
		return fileSelected;
	}

	/**
	 * The setter file selected function.
	 * @param filePath : the file
	 */
	public final void setFileSelected(final IFile filePath) {
		this.fileSelected = filePath;
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 * @param action the action proxy that handles the presentation
	 * portion of the action
	 * @param targetPart the new part target
	 */
	public final void setActivePart(final IAction action,
			final IWorkbenchPart targetPart) {
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 * @param action the action proxy that handles the presentation
	 * portion of the action
	 */
	public final void run(final IAction action) {
		Shell shell = new Shell();
		if (getFileSelected() == null) {
			//we cannot call the generator without selected file.
			// a message is sent to the user
			MessageDialog.openInformation(
					shell,
					"CimeroEditor Plug-in",
					"There is a problem with the selected file. The generation"
					+ "cannot be done !");
		} else {
			//we call the generator
			boolean res = false;
			try {
				DiagramEditor diag = CimeroRegistry.getInstance()
							.getDiagramEditor(getFileSelected().getName());
				if (diag != null) {
					Diagram diagram = diag.getDiagram();
					BufferedWriter writer = new BufferedWriter(new FileWriter("C:/tmp/exported.spagicml"));
					String diagramName = diagram.getCimeroDiagramName();
					writer.write("<SPAGIC diagramName='"+diagramName+"'>\n");
					List<GeneriqueCimeroModel> listModels = diagram.getLstCimeroObject();
					for (GeneriqueCimeroModel aModel : listModels){
						writer.write("<MODEL class='"+aModel.getClass().getName()+"' name='"+aModel.getName()+"'>\n");
						Set<String> propertiesKey = aModel.getPropertyKeys();
						writer.write("<PROPERTIES>\n");
						for (String pKey : propertiesKey){
							writer.write("<PROPERTY name='"+pKey+"' value='"+aModel.getPropertyValue(pKey)+"'/>\n");
						}
						if (aModel.getClass().getName().endsWith("Routeur")){
							Routeur rout = (Routeur)aModel;
							writer.write("<RULES>\n");
							List<Rule> listRules = rout.getMaLstRule();
							for (Rule rl : listRules){
								
								writer.write("<RULE xPathRule='"+rl.getXPathRule()+"' \n");
								writer.write(" valueRule='" + rl.getValueRule()+"' \n>");
								//writer.write(" valuePropertyName='" + rl.getValuePropertyName()+"' \n>");
								if (rl.getConn() != null){
									writer.write("<RULE_CON from='"+rl.getConn().getSource().getName()+"' to='"+rl.getConn().getTarget().getName()+"' />");
								}
								
								writer.write("</RULE>\n");
							
							}
							writer.write("</RULES>\n");
						}
						writer.flush();
						writer.write("</PROPERTIES>\n");
						// -- CONNECTIONS	
						writer.write("<CONNECTIONS>\n");
						writer.write("<INCOMING_CONNECTIONS number='"+aModel.getNbInputs()+"'>\n");
						List<Connection> incomings = aModel.getIncomingConnections();
						for (Connection aConnection : incomings){
							writer.write("<INCOMING_CONNECTION from='"+aConnection.getSource().getName()+"' to='"+aConnection.getTarget().getName()+"'/>\n");
							writer.flush();
						}
						writer.write("</INCOMING_CONNECTIONS>\n");
						
						writer.write("<OUTCOMING_CONNECTIONS number='"+aModel.getNbOutputs()+"'>\n");
						List<Connection> outcomings = aModel.getOutgoingConnections();
						for (Connection aConnection : outcomings){
							writer.write("<OUTCOMING_CONNECTION from='"+aConnection.getSource().getName()+"' to='"+aConnection.getTarget().getName()+"'/>\n");
							writer.flush();
						}
						writer.write("</OUTCOMING_CONNECTIONS>\n");
						writer.write("</CONNECTIONS>\n");
						writer.write("</MODEL>\n");
						writer.flush();
					}//-- end for
					writer.write("</SPAGIC>\n");
					writer.flush();
					writer.close();	
				} else {
					//we open the diagramEditor
					shell.getDisplay().syncExec(new Runnable() {
						public void run() {
							IWorkbenchPage page2 = null;
							page2 = PlatformUI.getWorkbench()
									.getActiveWorkbenchWindow().getActivePage();
							try {
								IDE.openEditor(page2, getFileSelected(), true);
							} catch (PartInitException e) {
								e.printStackTrace();
							}
						}
					});
					//and we call again this function
					run(action);
				}
			} catch (IOException ioe) {
				
				return;
			} catch (Exception e) {
				CimeroLog.errorLog(
						"JAR file generation exception"
						, e);
			}
		}
	}

	/**
	 * This function is called when the user selected the file in the package
	 * explorer.
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 * @param action the action proxy that handles the presentation
	 * 			portion of the action
	 * @param selection : the selected file
	 */
	public final void selectionChanged(final IAction action,
			final ISelection selection) {
		//we save the select file
		setFileSelected((IFile) ((StructuredSelection) selection)
				.getFirstElement());
	}
}
