package com.bull.cimero.pluginEditor.popupMenu;

import it.eng.utils.DialogUtils;
import it.eng.utils.ProcessModel;

import java.io.IOException;
import org.eclipse.core.resources.IFile;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import com.bull.cimero.pluginEditor.editors.CimeroRegistry;
import com.bull.cimero.pluginEditor.editors.DiagramEditor;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class RegisterServiceAssemblyAction implements IObjectActionDelegate {

	/** the selected file. */
	private IFile fileSelected = null;

	/**
	 * Constructor for GenerateJarFileAction.
	 */
	public RegisterServiceAssemblyAction() {
		super();
	}

	/**
	 * the getter fileselected function.
	 * @return IFile
	 */
	public final IFile getFileSelected() {
		return fileSelected;
	}

	/**
	 * The setter file selected function.
	 * @param filePath : the file
	 */
	public final void setFileSelected(final IFile filePath) {
		this.fileSelected = filePath;
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 * @param action the action proxy that handles the presentation
	 * portion of the action
	 * @param targetPart the new part target
	 */
	public final void setActivePart(final IAction action,
			final IWorkbenchPart targetPart) {
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 * @param action the action proxy that handles the presentation
	 * portion of the action
	 */
	public final void run(final IAction action) {
		Shell shell = new Shell();
		
		
//		MessageDialog.openInformation(
//				shell,
//				"CimeroEditor Plug-in Engineering Extensione",
//				"Register Service Assembly Action");
		
		if (getFileSelected() == null) {
			//we cannot call the generator without selected file.
			// a message is sent to the user
			MessageDialog.openInformation(
					shell,
					"CimeroEditor Plug-in",
					"There is a problem with the selected file.The registration of service assembly in db"
					+ "cannot be done !");
		} else {
			//we call the generator
			boolean res = false;
			try {
				DiagramEditor diag = CimeroRegistry.getInstance()
							.getDiagramEditor(getFileSelected().getName());
				if (diag != null) {
					
				
						
						ProcessModel processModel = diag.getProcessModel();
						//int status = diag.verifyServiceAssemblyStatus();
						
						
						
						res = diag.storeProcessModel(processModel);
					
					
				
					//
					if (!res) {
						DialogUtils.information(shell, "Fail occurs..");

					}else{
						DialogUtils.information(shell, "Process Published..");
					}
				} else {
					//we open the diagramEditor
					shell.getDisplay().syncExec(new Runnable() {
						public void run() {
							IWorkbenchPage page2 = null;
							page2 = PlatformUI.getWorkbench()
									.getActiveWorkbenchWindow().getActivePage();
							try {
								IDE.openEditor(page2, getFileSelected(), true);
							} catch (PartInitException e) {
								e.printStackTrace();
							}
						}
					});
					//and we call again this function
					run(action);
				}
			} catch (IOException ioe) {
				//user doesn't want to replace the existing file
				//CimeroLog.infoLog(
				//		"JAR file generation stopped");
				return;
			} catch (Exception e) {
				CimeroLog.errorLog(
						"Registration in DB Exception"
						, e);
			}
		}
	
	}

	/**
	 * This function is called when the user selected the file in the package
	 * explorer.
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 * @param action the action proxy that handles the presentation
	 * 			portion of the action
	 * @param selection : the selected file
	 */
	public final void selectionChanged(final IAction action,
			final ISelection selection) {
		//we save the select file
		setFileSelected((IFile) ((StructuredSelection) selection)
				.getFirstElement());
	}
}
