package com.bull.cimero.pluginEditor.utils.propertyCombo;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.views.properties.PropertyDescriptor;

/**
 * Descriptor for a property that has a value which should be edited with a
 * label cell editor.
 * <p>
 * This class may be instantiated; it is not intended to be subclassed.
 * </p>
 * <p>
 * Example:
 * <pre>
 * IPropertyDescriptor pd = new LabelPropertyDescriptor("surname", "Last Name");
 * </pre>
 * </p>
 */
public class LabelPropertyDescriptor extends PropertyDescriptor {
    /**
     * Creates an property descriptor with the given id and display name.
     *
     * @param id the id of the property
     * @param displayName the name to display for the property
     */
    public LabelPropertyDescriptor(final Object id,
    		final String displayName) {
        super(id, displayName);
    }

    /**
     * The <code>LabelPropertyDescriptor</code> implementation of this
     * <code>IPropertyDescriptor</code> method creates and returns a new
     * <code>TextCellEditor</code>.
     * <p>
     * The editor is configured with the current validator if there is one.
     * </p>
     * @param parent : the parent that contain this label
     * @return the cell editor.
     */
    public final CellEditor createPropertyEditor(final Composite parent) {
        CellEditor editor = new TextCellEditor(parent, SWT.READ_ONLY);
        if (getValidator() != null) {
			editor.setValidator(getValidator());
		}
        return editor;
    }
}
