package com.bull.cimero.pluginEditor.wizards;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.ICommand;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;
import org.eclipse.ui.ide.IDE;

import cimeroEditor.CimeroEditorPlugin;

import com.bull.cimero.pluginEditor.editors.CimeroRegistry;
import com.bull.cimero.pluginEditor.editors.DiagramEditor;
import com.bull.cimero.pluginEditor.editors.model.Connection;
import com.bull.cimero.pluginEditor.editors.model.Diagram;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.FTP;
import com.bull.cimero.pluginEditor.editors.model.BCModel.JMS;
import com.bull.cimero.pluginEditor.editors.model.BCModel.Mail;
import com.bull.cimero.pluginEditor.editors.model.BCModel.RSS;
import com.bull.cimero.pluginEditor.editors.model.BCModel.ScreenOutput;
import com.bull.cimero.pluginEditor.editors.model.BCModel.ServiceWeb;
import com.bull.cimero.pluginEditor.editors.model.BCModel.Timer;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Routeur;
import com.bull.cimero.pluginEditor.editors.model.BCModel.BindingComponentModel;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * @author Andrea Zoppello ( Engineering Ing. Informaztica )
 */
public class CimeroNewProjectWizard extends Wizard implements INewWizard {

	
	private WizardNewProjectCreationPage namePage;
	/** current selection of wizard. */
	private ISelection selection;

	/**
	 * Default constructor for CimeroNewWizard.
	 */
	public CimeroNewProjectWizard() {
		super();
		setNeedsProgressMonitor(true);
	}

	/**
	 * Adding the page to the wizard.
	 */
	public final void addPages() {
		try{
		super.addPages();
	      namePage = new WizardNewProjectCreationPage("New Integration Project Page");
	      namePage.setTitle("New Project");
	      namePage.setDescription("New Project Description");
//	      namePage.setImageDescriptor(ImageDescriptor.createFromFile(getClass(),
//	                                  "/org/ananas/xm/eclipse/resources/newproject58.gif"));
	      addPage(namePage);
	   	}
	   	catch(Exception x)
	   	{
	   		reportError(x);
	   	}
	   }

	
	public boolean performFinish()
	{
	   try
	   {
	      WorkspaceModifyOperation op =
	         new WorkspaceModifyOperation()
	      {
	         protected void execute(IProgressMonitor monitor)
	         {
	            createProject(monitor != null ?
	                          monitor : new NullProgressMonitor());
	         }
	      };
	      getContainer().run(false,true,op);
	   }
	   catch(InvocationTargetException x)
	   {
		  reportError(x);
	      return false;
	   }
	   catch(InterruptedException x)
	   {
		  reportError(x);
	      return false;
	   }
	   return true; 
	}
	

	   protected void createProject(IProgressMonitor monitor)
	   {
	      monitor.beginTask("Creating Project",50);
	      try
	      {
	         IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
	         monitor.subTask("Creating Project Directories ");
	         IProject project = root.getProject(namePage.getProjectName());
	         IProjectDescription description = ResourcesPlugin.getWorkspace().newProjectDescription(project.getName());
	         if(!Platform.getLocation().equals(namePage.getLocationPath()))
	            description.setLocation(namePage.getLocationPath());
	         /*
	         description.setNatureIds(new String[] { PluginConstants.NATURE_ID });
	         ICommand command = description.newCommand();
	         command.setBuilderName(PluginConstants.BUILDER_ID);
	         description.setBuildSpec(new ICommand[] { command });
	         */
	         project.create(description,monitor);
	         monitor.worked(10);
	         project.open(monitor);
	         /*
	         project.setPersistentProperty(PluginConstants.SOURCE_PROPERTY_NAME,"src");
	         project.setPersistentProperty(PluginConstants.RULES_PROPERTY_NAME,"rules");
	         project.setPersistentProperty(PluginConstants.PUBLISH_PROPERTY_NAME,"publish");
	         project.setPersistentProperty(PluginConstants.BUILD_PROPERTY_NAME,"false");
	         */
	         monitor.worked(10);
	         IPath projectPath = project.getFullPath();
	         IPath integrationProcessPath = projectPath.append("IntegrationProcess");
	         IPath semanticRulesPath = projectPath.append("SemanticRules");
	         IPath syntaxRulesPath = projectPath.append("SyntaxRules");
	         IPath scriptsPath = projectPath.append("Scripts");
	         IPath mappingsPath = projectPath.append("Mappings");
	         IPath wsdlFilesPath = projectPath.append("WsdlFiles");
	         
	         IFolder integrationProcessPathFolder = root.getFolder(integrationProcessPath);
	         IFolder semanticRulesPathFolder = root.getFolder(semanticRulesPath );
	         IFolder syntaxRulesPathFolder = root.getFolder(syntaxRulesPath);
	         IFolder scriptsPathPathFolder = root.getFolder(scriptsPath);
	         IFolder mappingsPathPathFolder = root.getFolder(mappingsPath);
	         IFolder wsdlFilesPathPathFolder = root.getFolder(wsdlFilesPath);
	              
	         
	         createFolderHelper(integrationProcessPathFolder, monitor);
	         createFolderHelper(semanticRulesPathFolder,monitor);
	         createFolderHelper(syntaxRulesPathFolder,monitor);
	         createFolderHelper(scriptsPathPathFolder,monitor);
	         createFolderHelper(mappingsPathPathFolder,monitor);
	         createFolderHelper(wsdlFilesPathPathFolder,monitor);
	         /*
	         monitor.subTask("Creating Files ");
	         
	         IPath indexPath = srcPath.append("index.xml"),
	               defaultPath = rulesPath.append("default.xsl");
	         IFile indexFile = root.getFile(indexPath),
	               defaultFile = root.getFile(defaultPath);
	         Class clasz = getClass();
	         InputStream indexIS = clasz.getResourceAsStream("/org/ananas/xm/eclipse/resources/index.xml"),
	                     defaultIS = clasz.getResourceAsStream("/org/ananas/xm/eclipse/resources/default.xsl");
	         indexFile.create(indexIS,false,new SubProgressMonitor(monitor,10));
	         defaultFile.create(defaultIS,false,new SubProgressMonitor(monitor,10));
	      	 */
	      }
	      catch(CoreException x)
	      {
	         reportError(x);
	      }
	      finally
	      {
	         monitor.done();
	      }
	   }

	   /**
	    * Helper method: it recursively creates a folder path.
	    * @param folder
	    * @param monitor
	    * @throws CoreException
	    * @see java.io.File#mkdirs()
	    */
	   private void createFolderHelper(IFolder folder,IProgressMonitor monitor)
	      throws CoreException
	   {
	      if(!folder.exists())
	      {
	         IContainer parent = folder.getParent();
	         if(parent instanceof IFolder
	            && (!((IFolder)parent).exists()))
	            createFolderHelper((IFolder)parent,monitor);
	         folder.create(false,true,monitor);
	      }
	   }

	/**
	 * method to throw real exception.
	 * @param message : error message
	 * @throws CoreException : the wanted exception
	 */
	private void throwCoreException(final String message) throws CoreException {
		IStatus status = null;
		status = new Status(
				IStatus.ERROR, "CimeroEditor",
				IStatus.OK, message, null);
		throw new CoreException(status);
	}

	/**
	 * We will accept the selection in the workbench to see if
	 * we can initialize from it.
	 * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 * @param workbench : the workbench
	 * @param selectionParam : current selection
	 */
	public final void init(
			final IWorkbench workbench,
			final IStructuredSelection selectionParam) {
		setNeedsProgressMonitor(true);
	}

	
	
	
	
	/**
	    * Displays an error that occured during the project creation.
	    * @param x details on the error
	    */
	   private void reportError(Exception x)
	   {
	      ErrorDialog.openError(getShell(),
	                            "Error",
	                            "Error in Creating New Project",
	                            makeStatus(x));
	   }
	   
	   
	   public static IStatus makeStatus(Exception x)
	   {
		   		
	         return new Status(IStatus.ERROR,
	                            CimeroEditorPlugin.pluginId,
	                            IStatus.ERROR,
	                            x.getMessage(), null);

	   }

	   

	

	
}
