package it.eng.cimero.plugin.dialogs;

import it.eng.cimero.plugin.providers.RulesContentProvider;
import it.eng.cimero.plugin.providers.RulesLabelProvider;

import it.eng.cimero.plugin.providers.ServiceAssemblyContentProvider;
import it.eng.cimero.plugin.providers.ServiceAssemblyLabelProvider;
import it.eng.spagosoa.metadb.api.IMetadbApi;
import it.eng.spagosoa.metadb.model.IterType;
import it.eng.spagosoa.metadb.model.Rule;
import it.eng.spagosoa.metadb.model.ServiceAssembly;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

public class ListServiceAssemblyDialog extends TitleAreaDialog {
	
    private SashForm internalSash  = null; 
    
    private Table tableSA = null;
    private TableViewer tbvSA = null; 
    private List<ServiceAssembly> theSAS = null;
    
    private ServiceAssembly selectedSA = null;
    
    private boolean closeOnDoubleClick = false;
    private IMetadbApi metadb = null;
    private String message = null;
	public ListServiceAssemblyDialog(Shell parentShell, IMetadbApi metadb, String message) {
		
		super(parentShell);
		this.theSAS = new ArrayList<ServiceAssembly>();
		this.metadb = metadb;
		
	}
	
	public ListServiceAssemblyDialog(Shell parentShell, IMetadbApi metadb, boolean closeOnDoubleClick, String message) {
		
		super(parentShell);
		this.theSAS = new ArrayList<ServiceAssembly>();
		this.metadb = metadb;
		this.closeOnDoubleClick = closeOnDoubleClick;
	}
	
	
				

	/**
	   * @see org.eclipse.jface.window.Window#create() We complete the dialog with
	   *      a title and a message
	   */
	  public void create() {
		super.create();
		setTitle("Business Processes");
		if (message != null)
			setMessage(message);
		else
			setMessage("Business Processes");
	  }

	 
	  

	@Override
	protected Control createContents(Composite parent) {
		Control ctrl = super.createContents(parent);
		return ctrl;
	}

	protected Control createDialogArea(Composite parent) {
		
        this.internalSash = new SashForm(parent,  SWT.VERTICAL | SWT.FLAT);
        this.internalSash.setLayoutData(new GridData(GridData.FILL_BOTH));
       	
    	final ListServiceAssemblyDialog theDialog = this;
    	
    	this.theSAS = getSAS(); 
    	
    	
        this.tableSA = new Table(internalSash, SWT.MULTI | SWT.FULL_SELECTION | SWT.BORDER);
        this.tableSA.setLinesVisible(true);
        this.tableSA.setHeaderVisible(true);
        TableColumn column = null;
		column = new  TableColumn(tableSA, SWT.LEFT);
		column.setText("Name");
		column.setWidth(200);
		column = new  TableColumn(tableSA, SWT.LEFT);
		column.setText("Description");
		column.setWidth(200);
		this.tableSA.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
        this.tbvSA = new TableViewer(tableSA);
        tbvSA.setContentProvider(new ServiceAssemblyContentProvider());
        tbvSA.setLabelProvider(new ServiceAssemblyLabelProvider());
        
    	tbvSA.setInput(theSAS);
        
    	
        
        tbvSA.addDoubleClickListener(new IDoubleClickListener(){
			public void doubleClick(DoubleClickEvent event) {
				
				if (closeOnDoubleClick){
					TableItem[] selection = tableSA.getSelection();
					if ((selection != null) && (selection.length > 0)){
            			selectedSA = (ServiceAssembly)((TableItem)selection[0]).getData();
            			close();
            		}else{
            			MessageDialog.openConfirm(
        						getShell(),
        						"CimeroEditor Plug-in",
        						"Select a service assembly ");
            		}
				}
				
			}});
        return parent; 
	
	}
	
	protected void okPressed() {
		super.okPressed();
		
	}
	
	@Override
	protected void buttonPressed(int buttonId) {
	
		super.buttonPressed(buttonId);
	}


	protected void createButtonsForButtonBar(Composite parent) {
		// create OK and Cancel buttons by default
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.CLOSE_LABEL,
				true);
		
	}

	
	
	



	public List<ServiceAssembly> getSAS(){
		boolean needToHandleOpenClose = !metadb.isOpen();
		if (needToHandleOpenClose)
			metadb.open();
		List<ServiceAssembly> result = metadb.getServiceAssembliesLastVersion();
		if (needToHandleOpenClose)
			metadb.close();
		return result;
	}

	public ServiceAssembly getSelectedSA() {
		return selectedSA;
	}

	public void setSelectedSA(ServiceAssembly selectedSA) {
		this.selectedSA = selectedSA;
	}
	
	



	
	
}
