package it.eng.components.cimero;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Transformeur;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.SE.SELWFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.SE.TransformeurFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.LWBeanClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;

/**
 * This class will be use to add in the ServiceMix model, an activation
 * spec that contain a transformeur.
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class GenericGroovyFactory extends SELWFactory {

	/**the class using by ServiceMix to the transformer.*/
	public static final String CLASS =
		"org.apache.servicemix.components.groovy.GroovyComponent";
	/**The property name to set the xslt file.*/
	private static final String GROOVYRESOURCE = "script";

	/**
	 * default constructor.
	 * @param ref : the reference of the current namespace.
	 */
	public GenericGroovyFactory(final NameSpaceClass ref) {
		super(ref);
	}
	
	/**
	 * this function will create an activation spec that represents
	 * a transformeur in serviceMix.
	 * @param elt : the element using to create this activation spec.
	 * @param force : false if the validation must to be do.
	 * @return the activation spec create.
	 * @throws Exception
	 * @throws Exception : if the is a error during generation
	 */
	public final ActivationSpecClass createXMLModel(
			final GeneriqueCimeroModel elt,
			final Boolean force) throws Exception {
		ActivationSpecClass myActi = null;
		//if not null and valide and instance of transformer
		if ((elt != null) && (elt instanceof GenericGroovyComponent)) {
			if ((elt.isValide(false)) || (force)) {
				GenericGroovyComponent monTrfs = (GenericGroovyComponent) elt;
				myActi = new ActivationSpecClass();
				myActi.setNameSpaceRef(getNamespaceRef());
				myActi.setComponentName(monTrfs.getName());
				myActi.setServiceName(monTrfs.getName(),
						new NameSpaceClass(NAMESPACENAME , NAMESPACEVALUE));
				setDestinationService(myActi , elt);
				//add the component
				ComponentClass myCompo = createComponent(monTrfs);
				myActi.setComponent(myCompo);
				return myActi;
			}
		}
		throw new Exception("The generique cimero object is not valide");

	}

	/**
	 * This function return the servicemix component of a Transformeur.
	 * @param monTrfs : the cimero transformeur
	 * @return the servicemix component.
	 */
	private ComponentClass createComponent(final GenericGroovyComponent monTrfs) {
		//create the componentclass instance
		ComponentClass monCompo = new ComponentClass(getNamespaceRef());
		//add the bean class.
		LWBeanClass monB = createBeanTransformeur(monTrfs);
		monCompo.setMonBean(monB);
		return monCompo;
	}

	/**
	 * This function creates the bean using by the transformeur in the
	 * serviceMix model.
	 * @param monTrfs : the cimero element
	 * @return the bean transformer.
	 */
	private LWBeanClass createBeanTransformeur(final GenericGroovyComponent monTrfs) {
		LWBeanClass myBean = new LWBeanClass(CLASS);
		String relativePath =  null; 
		String originalResourcePath = monTrfs.getGroovyPath();
		
		if (originalResourcePath.startsWith("file:"))
			relativePath = originalResourcePath.substring("file:".length());
		else
			relativePath = originalResourcePath.substring("project:".length());
		relativePath = relativePath.replace("\\", "/");
		if (relativePath.lastIndexOf("/") > 0){
			relativePath = relativePath.substring(relativePath.lastIndexOf("/") + 1);
		}
		
		String classPathStyleResource = "classpath:"+ relativePath;
		myBean.addProperty(createProperty(GROOVYRESOURCE , classPathStyleResource)
				, "");
		return myBean;
	}
}

