package it.eng.components.cimero;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Transformeur;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.SE.SELWFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.SE.TransformeurFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.LWBeanClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;

/**
 * This class will be use to add in the ServiceMix model, an activation
 * spec that contain a transformeur.
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class HL7ComponentFactory extends SELWFactory {

	/**the class using by ServiceMix to the transformer.*/
	public static final String CLASS =
		"it.eng.dorsale.sanita.HL7Component";
	/**The property name to set the xslt file.*/
	public static String TRANSFORMATION_NAME = "transformationName";
	public static String TRANSFORMATION_CLASS = "transformationClass";
	public static String TRANSFORMATION_TYPE = "transformationType";

	

	/**
	 * default constructor.
	 * @param ref : the reference of the current namespace.
	 */
	public HL7ComponentFactory(final NameSpaceClass ref) {
		super(ref);
	}
	
	/**
	 * this function will create an activation spec that represents
	 * a transformeur in serviceMix.
	 * @param elt : the element using to create this activation spec.
	 * @param force : false if the validation must to be do.
	 * @return the activation spec create.
	 * @throws Exception
	 * @throws Exception : if the is a error during generation
	 */
	public final ActivationSpecClass createXMLModel(
			final GeneriqueCimeroModel elt,
			final Boolean force) throws Exception {
		ActivationSpecClass myActi = null;
		//if not null and valide and instance of transformer
		if ((elt != null) && (elt instanceof HL7Component)) {
			if ((elt.isValide(false)) || (force)) {
				HL7Component monHL7 = (HL7Component) elt;
				myActi = new ActivationSpecClass();
				myActi.setNameSpaceRef(getNamespaceRef());
				myActi.setComponentName(monHL7.getName());
				myActi.setServiceName(monHL7.getName(),
						new NameSpaceClass(NAMESPACENAME , NAMESPACEVALUE));
				setDestinationService(myActi , elt);
				//add the component
				ComponentClass myCompo = createComponent(monHL7);
				myActi.setComponent(myCompo);
				return myActi;
			}
		}
		throw new Exception("The generique cimero object is not valide");

	}

	/**
	 * This function return the servicemix component of a Transformeur.
	 * @param monTrfs : the cimero transformeur
	 * @return the servicemix component.
	 */
	private ComponentClass createComponent(final HL7Component monHL7) {
		//create the componentclass instance
		ComponentClass monCompo = new ComponentClass(getNamespaceRef());
		//add the bean class.
		LWBeanClass monB = createBeanTransformeur(monHL7);
		monCompo.setMonBean(monB);
		return monCompo;
	}

	/**
	 * This function creates the bean using by the transformeur in the
	 * serviceMix model.
	 * @param monTrfs : the cimero element
	 * @return the bean transformer.
	 */
	private LWBeanClass createBeanTransformeur(final HL7Component monHL7) {
		LWBeanClass myBean = new LWBeanClass(CLASS);
	
		myBean.addProperty(createProperty(TRANSFORMATION_NAME , monHL7.getTransformationName()), "");
		myBean.addProperty(createProperty(TRANSFORMATION_TYPE , monHL7.getTransformationType()), "");
		myBean.addProperty(createProperty(TRANSFORMATION_CLASS , monHL7.getTransformationClass()), "");
		return myBean;
	}
}

