package it.eng.utils;

import it.eng.spagosoa.metadb.api.IMetadbApi;
import it.eng.spagosoa.metadb.model.SaFlow;
import it.eng.spagosoa.metadb.model.ServiceAssembly;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.bull.cimero.pluginEditor.utils.log.CimeroLog;
/**
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class ProcessModel {
	
	public static final int PROCESS_IS_NOT_IN_DATABASE = 1;
	public static final int PROCESS_IS_IN_DATABASE_WITH_SAME_STRUCTURE = 2;
	public static final int PROCESS_IS_IN_DATABASE_WITH_DIFFERENT_STRUCTURE = 3;
	
	private String processName = null;
	private String processDescription  = null;
	private int status = 1;
	
	private Map<String, EndpointModel> endpointMap = null;
	private List<String> transitionList = null;
	
	private List<String> startEndpointNames = null;
	private List<String> endEndpointNames = null;
	
	private List<String> targets = null;
	
	public ProcessModel(){
	
		endpointMap = new HashMap<String, EndpointModel>();
		transitionList = new ArrayList<String>();
		startEndpointNames = new ArrayList<String>();
		endEndpointNames = new ArrayList<String>();
		targets = new ArrayList<String>();
	}
	
	public ProcessModel(Map<String, EndpointModel> endpointMap, List<String> transitionList){
		
		this.endpointMap = endpointMap;
		this.transitionList = transitionList;
	
	}
	
	public void addEndpoint(String namespace, String endpointName, String componentName, String typeService ){
		EndpointModel epModel = new EndpointModel(namespace, endpointName, componentName, typeService);
		this.endpointMap.put(endpointName, epModel);
	}
	
	public void addEndpointWithProperty(String namespace, String endpointName, String componentName,String typeService, PropertyModel prop ){
		List<PropertyModel> properties = new ArrayList<PropertyModel>();
		properties.add(prop);
		addEndpointWithProperties(namespace, endpointName, componentName,typeService, properties);
		
	}
	public void addEndpointWithProperties(String namespace, String endpointName, String componentName, String typeService, List<PropertyModel> properties){
		EndpointModel epModel = new EndpointModel(namespace, endpointName, componentName,typeService, properties);
		this.endpointMap.put(endpointName, epModel);
		CimeroLog.infoLog("-> Process ["+processName+"] Added Endpoint: ["+namespace+":"+endpointName+"] for Component ["+componentName+"]" );
	}
	
	public void addFlow(String epNameSource, String epNameTarget ){
		if ((epNameSource != null) && 
				((epNameTarget ==  null) || (epNameTarget.trim().length() == 0)) ){
				endpointMap.get(epNameSource).setEnd(true);
				endEndpointNames.add(epNameSource);
				return;
		}
		CimeroLog.infoLog("-> Process ["+processName+"] Registered Flow: ["+epNameSource+"]->["+epNameTarget+"]");
		transitionList.add("["+epNameSource+"]->["+epNameTarget+"]");
		targets.add(epNameTarget);
	}
	
	public void calculateStartEndpoints(){
		CimeroLog.infoLog("-> Calculating Starting Endpoint for ["+processName+"]");
		Set<String> endpointSet = endpointMap.keySet();
		for (String epName : endpointSet ){
			if (!targets.contains(epName)){
				this.endpointMap.get(epName).setStart(true);
				this.startEndpointNames.add(epName);
				CimeroLog.infoLog("-> Start Endpoint for ["+processName+"] is ["+epName+"]");
			}
		}
	}

	public int getStatus() {
		return status;
	}

	public void setStatus(int status) {
		this.status = status;
	}
	
	/*
	
	*/

			
			
			
			
			
			
			public Map<String, EndpointModel> getEndpointMap() {
				return endpointMap;
			}

			public void setEndpointMap(Map<String, EndpointModel> endpointMap) {
				this.endpointMap = endpointMap;
			}

			public String getProcessDescription() {
				return processDescription;
			}

			public void setProcessDescription(String processDescription) {
				this.processDescription = processDescription;
			}

			public String getProcessName() {
				return processName;
			}

			public void setProcessName(String processName) {
				this.processName = processName;
			}

			public List<String> getTransitionList() {
				return transitionList;
			}

			public void setTransitionList(List<String> transitionList) {
				this.transitionList = transitionList;
			}
			
}
