package it.eng.utils;


import it.eng.graphviz.GEndNode;
import it.eng.graphviz.GFlow;
import it.eng.graphviz.GNode;
import it.eng.graphviz.GStartNode;
import it.eng.graphviz.GraphViz;
import it.eng.spagosoa.metadb.api.IMetadbApi;
import it.eng.spagosoa.metadb.model.Endpoint;
import it.eng.spagosoa.metadb.model.IterType;
import it.eng.spagosoa.metadb.model.Property;
import it.eng.spagosoa.metadb.model.SaFlow;
import it.eng.spagosoa.metadb.model.ServiceAssembly;
import it.eng.spagosoa.metadb.model.TypeService;

import java.io.ByteArrayOutputStream;
import java.io.StringWriter;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.LogFactory;

public class ProcessModelsUtils {
	
	private ILogger logger = null;
	private String  graphVizExeLocation = null;
	private String  graphVizTempFolder = null;
	
	public ProcessModelsUtils(ILogger logger, String graphVizExeLocation, String graphVizTempFolder ){
		
		this.logger = logger;
		this.graphVizExeLocation = graphVizExeLocation;
		this.graphVizTempFolder = graphVizTempFolder;
		
		logger.info(" Process Model Utils   -> Constructor");
		logger.info(" GraphViz Dot Location -> ["+graphVizExeLocation+"]");
		logger.info(" GraphViz tmp folder -> ["+graphVizTempFolder+"]");
	}
	
	public ProcessModelsUtils(){
		this(new CommonsLoggingILoggerAdapter(LogFactory.getLog(ProcessModelsUtils.class)),
						   "C:/Programmi/ATT/Graphviz/bin/dot", 
						   "C:/tmp");
		
	}
	/**
	 * @param metadb - The metadbapi
	 * @param saInDb - Service Assembly in database
	 * @return the ProcessModel object rappresentation for the Service Assembly Stored
	 * 
	 */
	public ProcessModel getLastVersionStoredProcessModel(IMetadbApi metadb, ServiceAssembly saInDb){
		ProcessModel processModel = new ProcessModel();
		
		List<Endpoint> storedEndpointObjectList = metadb.getEndpointsForServiceAssembly(saInDb);
		
		for (Endpoint ep : storedEndpointObjectList){
			processModel.addEndpoint(ep.getNamespace(), ep.getName(), ep.getComponentName(), ep.getTypeService().getValue());
		}
		
		List<SaFlow> storedFlowsObject = metadb.getSaFlowsForServiceAssembly(saInDb);
		
		String endpointSource = null;
		String endpointDest = null;
	
		
		for (SaFlow saFlow : storedFlowsObject){
			endpointSource = saFlow.getEndpointByIdEndpointSource().getName();
			endpointDest = saFlow.getEndpointByIdEndpointTarget().getName();
			processModel.addFlow(endpointSource, endpointDest);
		}
		
		processModel.calculateStartEndpoints();
		return processModel;
	
	}
	
	public void storeProcessStructure(IMetadbApi metadb, ProcessModel aProcessModel, ServiceAssembly aServiceAssembly) throws Throwable{
		
		Map<String, EndpointModel> endpointMap = aProcessModel.getEndpointMap();
		List<String> flows = aProcessModel.getTransitionList();
		
		Set<String> setEndpointNames = endpointMap.keySet();
		EndpointModel epModel = null;
		for (String e : setEndpointNames ){
			this.logger.info("Storing ["+e+"] for Process["+aProcessModel.getProcessName()+" ver. "+aServiceAssembly.getSaVersion()+"]");
			epModel = endpointMap.get(e); 
			Endpoint ep = createEndpointFromEndpointModel(metadb, aServiceAssembly,epModel);
			metadb.storeEndPoint(ep);
			
			for (PropertyModel pm : epModel.getProperties()){
				Property prop = createPropertyFromPropertyModel(pm);
				if (prop != null){
					metadb.storePropertyAndLinkToEndpoint(prop, ep);
				}else{
					this.logger.warning("Property ["+pm.getName() +"] of endpoint ["+epModel.getEndpointname()+"] IS NULL");
				}
			}
		}
		String source = null;
		String target = null;
		String[] tokens = null;
		for (String flow : flows ){
			tokens = flow.split("->");
			source = tokens[0].substring(1, tokens[0].length() -1);
			target = tokens[1].substring(1, tokens[1].length() -1);
			System.out.println("Source ->" + source);
			System.out.println("Source ->" + target);
			metadb.storeFlow(aServiceAssembly, source, target);
		}
	}
	
	
	public String getSvgForSa(IMetadbApi meta, ServiceAssembly sa) {
		try{
			GraphViz gv = new GraphViz("graph"+System.currentTimeMillis());
			gv.setDotExeLocation(getGraphVizExeLocation());
			gv.setTempDirLocation(getGraphVizTempFolder());
			gv.startGraph();
			List eps = meta.getEndpointsForServiceAssembly(sa);
			Endpoint ep = null;
			for (Iterator it = eps.iterator(); it.hasNext();){
				ep = (Endpoint)(it.next());
				if (ep.isIsStart()){
					gv.addNode(new GStartNode(String.valueOf(ep.getIdEndpoint()), String.valueOf(ep.getIdEndpoint())));
				}else if (ep.isIsEnd()){
					gv.addNode(new GEndNode(String.valueOf(ep.getIdEndpoint()), String.valueOf(ep.getIdEndpoint())));
				}else{
					gv.addNode(new GNode(String.valueOf(ep.getIdEndpoint()),String.valueOf(ep.getIdEndpoint())));
				}
			}
			List<SaFlow> flows = meta.getSaFlowsForServiceAssembly(sa);
			SaFlow saFlow = null;
			for (Iterator it = flows.iterator(); it.hasNext();){
				saFlow = (SaFlow)(it.next());
				gv.addFlow(new GFlow(String.valueOf(saFlow.getId().getIdEndpointSource()),
							 String.valueOf(saFlow.getId().getIdEndpointTarget())));
			}
			gv.endGraph();
		
			byte[] img = gv.getGraph(gv.getDotSource());
			StringWriter sw = new StringWriter();
			String svg = null;
		
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			bos.write(img);
			bos.flush();
			svg = bos.toString();
			bos.close();
		
		
			// Clean the DTD SVG Declartion
			
			String svgPart = svg.substring(svg.indexOf("<svg"));
			String svgWithoutDTD = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + svgPart;
			return svgWithoutDTD;
		}catch(Exception e){
			this.logger.error("Exception in getting SVG For Process"+sa.getName(), e);		
			return "";
		}	
		
	}
	
	
	
	public void storeProcessModel(IMetadbApi metadb, ProcessModel aProcessModel) throws Throwable{
		this.logger.info("-> Storing ProcessModel ["+aProcessModel.getProcessName()+"]");
		try{
			metadb.openTx();
				
			ServiceAssembly sa = null;
			if (aProcessModel.getStatus() == ProcessModel.PROCESS_IS_NOT_IN_DATABASE){
				this.logger.info("-> Process ["+aProcessModel.getProcessName()+"] IS NOT IN DATABASE");
				sa = createNewSa(metadb, aProcessModel.getProcessName(), aProcessModel.getProcessDescription());
				
				this.logger.info("-> Created The First Version for Process ["+aProcessModel.getProcessName()+"]");
				this.logger.info("-> Storing The First Version for Process ["+aProcessModel.getProcessName()+"] Structure ");
				storeProcessStructure(metadb,aProcessModel, sa);
				this.logger.info("-> Storing SVG For Process ["+aProcessModel.getProcessName()+"]");
				String svg = getSvgForSa(metadb, sa);
				if (svg != null && svg.trim().length() > 0 ){
					metadb.storeSvgForSA(sa, svg);
				}else{
					this.logger.warning("-> SVG Not Stored ["+aProcessModel.getProcessName()+"] results in a empty image " );	
				}
				this.logger.info("-> Finished to store ["+aProcessModel.getProcessName()+"] With Success " );
			}else if (aProcessModel.getStatus() == ProcessModel.PROCESS_IS_IN_DATABASE_WITH_DIFFERENT_STRUCTURE){
				this.logger.info("-> Process ["+aProcessModel.getProcessName()+"] IS ALREADY IN DATABASE WITH DIFFERENT STRUCTURE ");
				ServiceAssembly saInDatabase = metadb.getServiceAssemblyByName(aProcessModel.getProcessName());
				
				this.logger.info("-> In Database  Process ["+aProcessModel.getProcessName()+" ver. "+saInDatabase.getSaVersion()+"]");
				//
				// Set the end date in the current version of the sa
				//
				this.logger.info("-> Invalidating ["+aProcessModel.getProcessName()+" ver. "+saInDatabase.getSaVersion()+"] setting end date ");
				Date current = new Date();
				saInDatabase.setEnddate(current);
				
				
				sa = createNewSaVersion(metadb,  aProcessModel.getProcessName(), aProcessModel.getProcessDescription(), saInDatabase);
				this.logger.info("-> Created New Version ["+aProcessModel.getProcessName()+" ver. "+sa.getSaVersion()+"]");
				this.logger.info("-> Storing New Version ["+aProcessModel.getProcessName()+" ver. "+sa.getSaVersion()+"] Structure ");
				storeProcessStructure(metadb,aProcessModel, sa);
				
				this.logger.info("-> Copying Rules Form ["+aProcessModel.getProcessName()+" ver. "+saInDatabase.getSaVersion()+"] to ["+aProcessModel.getProcessName()+" ver. "+sa.getSaVersion()+"]");
				copyRules(metadb, saInDatabase, sa);

				//
				// Adjusting Iter Type Relation
				//
				this.logger.info("-> Update the reference in Iter Type Realtions ");
				List<IterType> itTypes = metadb.getIterTypeForServiceAssembly(saInDatabase);
				for (IterType it : itTypes){
					this.logger.info("-> Remove ["+saInDatabase.getName()+" ver. "+saInDatabase.getSaVersion()+"] from Iter ["+it.getName()+"]");
					metadb.unlinkServiceAssemblyIterType(saInDatabase, it);
					this.logger.info("-> Add ["+sa.getName()+" ver. "+sa.getSaVersion()+"] to Iter ["+it.getName()+"]");
					metadb.linkServiceAssemblyToIterType(sa, it);
					
				}
				this.logger.info("-> Storing SVG For Process ["+aProcessModel.getProcessName()+ " ver. "+sa.getSaVersion()+"]");
				String svg = getSvgForSa(metadb, sa);
				if (svg != null && svg.trim().length() > 0 ){
					metadb.storeSvgForSA(sa, svg);
				}else{
					this.logger.warning("-> SVG Not Stored ["+aProcessModel.getProcessName()+"] results in a empty image " );	
				}
				this.logger.info("-> Versioning Finished With Success " );
			}if (aProcessModel.getStatus() == ProcessModel.PROCESS_IS_IN_DATABASE_WITH_SAME_STRUCTURE){
				this.logger.info("-> Process ["+aProcessModel.getProcessName()+"] IS ALREADY IN DATABASE WITH DIFFERENT STRUCTURE ");
				ServiceAssembly saInDatabase = metadb.getServiceAssemblyByName(aProcessModel.getProcessName());
				List<Endpoint> eps = metadb.getEndpointsForServiceAssembly(saInDatabase);
				Map<String, EndpointModel> endpointMap = aProcessModel.getEndpointMap();
				EndpointModel epModel = null;
				for (Endpoint ep : eps){
					//
					// Removing all property
					//
					metadb.deleteAllEndpointProperty(ep);
					
					//
					// Recreating Properties
					//
					epModel = endpointMap.get(ep.getName());
					for (PropertyModel pm : epModel.getProperties()){
						Property prop = createPropertyFromPropertyModel(pm);
						if (prop != null){
							metadb.storePropertyAndLinkToEndpoint(prop, ep);
						}else{
							this.logger.warning("Property ["+pm.getName() +"] of endpoint ["+epModel.getEndpointname()+"] IS NULL");
						}
					}
					
				}
			}
			
			metadb.commitAndClose();

		}catch (Throwable e) {
			metadb.rollBackAndClose();
			throw e;
		}
	
	}
	
	protected void copyRules(IMetadbApi metadb, ServiceAssembly saPrevVersion, ServiceAssembly saNewVersion ){
		List<Endpoint> endpointOfPreviousVersion = metadb.getEndpointsForServiceAssembly(saPrevVersion);
		
		for(Endpoint prevEp : endpointOfPreviousVersion ){
			
			this.logger.info(" Verify if the new version of SA ["+saNewVersion.getName()+"] has an endpoint called ["+prevEp.getName()+"]");
			
			// If the new version of the sa has the same endpoint we must copy the rules
			Endpoint newVersionEp = metadb.getEndpointByName(saNewVersion, prevEp.getName());
			if (newVersionEp != null){
				this.logger.info(" The new version of SA ["+saNewVersion.getName()+"] HAS AN endpoint called ["+prevEp.getName()+"] WE MUST COPY RULES ");
				List<it.eng.spagosoa.metadb.model.Rule> rules = metadb.getRulesForEndpoint(prevEp);
				for (it.eng.spagosoa.metadb.model.Rule rule : rules){
					metadb.linkRuleToEndpoint(rule, newVersionEp);
				}
			}else{
				this.logger.info(" The new version of SA ["+saNewVersion.getName()+"] DOES NOT HAS AN endpoint called ["+prevEp.getName()+"]");
			}
		}
	}
	
	protected ServiceAssembly createNewSa(IMetadbApi metadb, String processName, String ProcessDescription){
		TypeService ts = metadb.getTypeServiceWithValue("GENERIC");
	
		ServiceAssembly aServiceAssembly = new ServiceAssembly();
		aServiceAssembly.setTypeService(ts);
		aServiceAssembly.setName(processName);
		aServiceAssembly.setDescription("Description for " +processName);
		aServiceAssembly.setStartdate(new Date());
		aServiceAssembly.setState(1);
		aServiceAssembly.setSaVersion(1);
	
		metadb.storeServiceAssembly(aServiceAssembly);
		return aServiceAssembly;
	}

	protected ServiceAssembly createNewSaVersion(IMetadbApi metadb, String processName, String ProcessDescription, ServiceAssembly saInDatabase){
		
		ServiceAssembly aServiceAssembly = new ServiceAssembly();
		aServiceAssembly.setTypeService(saInDatabase.getTypeService());
		aServiceAssembly.setName(processName);
		aServiceAssembly.setDescription(processName);
		aServiceAssembly.setStartdate(saInDatabase.getEnddate());
		aServiceAssembly.setState(saInDatabase.getState());
		aServiceAssembly.setSaVersion(saInDatabase.getSaVersion() + 1);
		aServiceAssembly.setUddiName(saInDatabase.getUddiName());
		aServiceAssembly.setUddiOrg(saInDatabase.getUddiOrg());
		
		metadb.storeServiceAssembly(aServiceAssembly);
		return aServiceAssembly;
	}
	
	protected Endpoint createEndpointFromEndpointModel(IMetadbApi metadb, ServiceAssembly aServiceAssembly, EndpointModel epModel){
		Endpoint ep = new Endpoint();
		
		TypeService ts = metadb.getTypeServiceWithValue(epModel.getTypeService());
		ep.setComponentName(epModel.getComponentName());
		ep.setComponentDescription("Description for Component ["+ep.getComponentName()+"]");
		ep.setServiceAssembly(aServiceAssembly);
		ep.setName(epModel.getEndpointname());
		ep.setNamespace(epModel.getNamespace());
		ep.setSaVersion(aServiceAssembly.getSaVersion());
		ep.setDescription("Description for ["+epModel.getEndpointname()+"]");
		ep.setIsEnd(epModel.isEnd());
		ep.setIsStart(epModel.isStart());
		ep.setTypeService(ts);
		return ep;
	}
	
	protected Property createPropertyFromPropertyModel(PropertyModel pm){
		Property prop = null;
		if (pm.getValue() != null){
			prop = new Property();
			prop.setCode(pm.getName());
			prop.setValue(pm.getValue());
		}
		
		return prop;
	}
	
	public void calculateProcessModelStatus(ProcessModel aProcessModel, IMetadbApi metadb){
		ServiceAssembly saInDb = metadb.getServiceAssemblyByName(aProcessModel.getProcessName());
		if (saInDb == null){
			aProcessModel.setStatus(ProcessModel.PROCESS_IS_NOT_IN_DATABASE);
			return;
		}else{
			// Check if endointList is the same
			ProcessModel lastVersionStoredProcessModel = getLastVersionStoredProcessModel(metadb, saInDb);
			
			Set<String> thisEndpoints = aProcessModel.getEndpointMap().keySet();
			Set<String> lastVersionStoredEndpoints = lastVersionStoredProcessModel.getEndpointMap().keySet();
			
			List<String> thisTransitions = aProcessModel.getTransitionList();
			List<String> lastVersionStoredTransitions = lastVersionStoredProcessModel.getTransitionList();
			
			if (   (thisEndpoints.size() != lastVersionStoredEndpoints.size()) 
				|| (thisTransitions.size() != lastVersionStoredTransitions.size())) {
				// There are differences in the number of endpoints
				aProcessModel.setStatus(ProcessModel.PROCESS_IS_IN_DATABASE_WITH_DIFFERENT_STRUCTURE);
				return;
				
			}else{
				//
				// There is the same number of endpoint, we must verify that are exactly the same endpoint
				// in both directions
				//
				for (String inferredEndpoint : thisEndpoints){
					
					if (!lastVersionStoredEndpoints.contains(inferredEndpoint)){
						// The number of the endpoint are the same but there is one endpoint in the
						// model that is not stored in database
						aProcessModel.setStatus(ProcessModel.PROCESS_IS_IN_DATABASE_WITH_DIFFERENT_STRUCTURE);
						return;
						
					}
				}
				for (String storedEndpoint : lastVersionStoredEndpoints){
					System.out.println("Verify if endpoint ["+storedEndpoint+"] is in structure");
					if (!thisEndpoints.contains(storedEndpoint)){
						// The number of the endpoint are the same but there is one endpoint in the
						// model that is not stored in database
						aProcessModel.setStatus(ProcessModel.PROCESS_IS_IN_DATABASE_WITH_DIFFERENT_STRUCTURE);
						return;
						
					}
				}
				
				//
				// There is the same number of flows, we must verify that are exactly the same endpoint
				// in both directions
				//
				 
				for (String flStr : thisTransitions){
					System.out.println("Verify if flStr ["+flStr+"] is in database");
					if (!lastVersionStoredTransitions.contains(flStr)){
						aProcessModel.setStatus(ProcessModel.PROCESS_IS_IN_DATABASE_WITH_DIFFERENT_STRUCTURE);
						return;
						
					}
				}
				for (String flStr : lastVersionStoredTransitions){
					System.out.println("Verify if flStr ["+flStr+"] is in current model");
					if (!thisTransitions.contains(flStr)){
						aProcessModel.setStatus(ProcessModel.PROCESS_IS_IN_DATABASE_WITH_DIFFERENT_STRUCTURE);
						return;
						
					}
				}
				aProcessModel.setStatus(ProcessModel.PROCESS_IS_IN_DATABASE_WITH_SAME_STRUCTURE);
				return;
			}
		}
	}
	
	public static PropertyModel createPropertyModel(String propName, String propValue){
		PropertyModel prop = new PropertyModel();
		prop.setName(propName);
		prop.setValue(propValue);
		return prop;
	}
	
	public ILogger getLogger() {
		return logger;
	}
	public void setLogger(ILogger logger) {
		this.logger = logger;
	}
	public String getGraphVizExeLocation() {
		return graphVizExeLocation;
	}
	public void setGraphVizExeLocation(String graphVizExeLocation) {
		this.graphVizExeLocation = graphVizExeLocation;
	}
	public String getGraphVizTempFolder() {
		return graphVizTempFolder;
	}
	public void setGraphVizTempFolder(String graphVizTempFolder) {
		this.graphVizTempFolder = graphVizTempFolder;
	}
	
	
}
